'use strict';

/**
 * 企业管理控制器
 *
 * @author CaiAoLin
 * @date 2017/11/8
 * @version
 */

const cldOfficeConst = require('../const/cld_office');
module.exports = app => {

    class EnterpriseController extends app.BaseController {

        /**
         * 企业列表
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async index(ctx) {
            const page = ctx.page;

            // 过滤数据
            ctx.service.enterprise.searchFilter(ctx.request.query);
            // 获取企业列表
            const enterpriseList = await ctx.service.enterprise.getListWithBuilder();
            const total = await ctx.service.enterprise.count();

            if (enterpriseList !== null) {
                // 整理数据
                let i = 0;
                for (const enterprise of enterpriseList) {
                    const manager = await ctx.service.manager.getDataById(enterprise.manager_id);
                    enterpriseList[i].manager_username = manager !== null ? manager.username : '';
                    const project_num = await ctx.service.project.count({ enterprise_id: enterprise.id });
                    enterpriseList[i].project_num = project_num;
                    i++;
                }
            }

            // 分页相关
            const pageInfo = {
                page,
                total: Math.ceil(total / app.config.pageSize),
                queryData: JSON.stringify(ctx.urlInfo.query),
            };

            // 下拉菜单
            const dropDownMenu = [
                { name: '新增企业', url: ctx.controllerName + '/add' },
            ];
            const renderData = {
                enterpriseList,
                pageInfo,
                dropDownMenu,
                displayStatus: ctx.service.enterprise.displayStatus,
                statusClass: ctx.service.enterprise.statusClass,
                officeList: JSON.stringify(cldOfficeConst.list),
                officeList2: cldOfficeConst.list,
            };
            await this.layout('enterprise/index.ejs', renderData);
        }

        /**
         * 新增企业页面
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async add(ctx) {
            // 获取数据规则
            const rule = ctx.service.enterprise.rule('enterprise');
            const frontRule = ctx.helper.validateConvert(rule);
            const managerList = await ctx.service.manager.getManagerList('office', false);

            // 面包屑
            let breadCrumb = [
                { name: ctx.topPermission.name },
                { name: '企业版客户', url: ctx.controllerName },
                { name: '添加企业' },
            ];
            breadCrumb = ctx.helper.convertBreadCrumb(breadCrumb, 'replace');
            const renderData = {
                rule: JSON.stringify(frontRule),
                officeList: JSON.stringify(cldOfficeConst.list),
                managerList: JSON.stringify(managerList),
                breadCrumb,
            };
            await this.layout('enterprise/add.ejs', renderData);
        }

        /**
         * 修改企业页面
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async modify(ctx) {
            // 获取数据规则
            const rule = ctx.service.enterprise.rule('enterpriseModify');
            const frontRule = ctx.helper.validateConvert(rule);

            let id = ctx.params.id;
            id = parseInt(id);

            try {
                if (isNaN(id) || id <= 0) {
                    throw '参数错误';
                }
                // 查找数据
                const enterpriseData = await ctx.service.enterprise.getDataById(id);
                // 项目数据存入session
                ctx.session.sessionEnterpriseData = enterpriseData;
                const managerList = await ctx.service.manager.getManagerList('office', false);

                // 面包屑
                const breadCrumb = this._getBreadCrumb(enterpriseData);
                const renderData = {
                    rule: JSON.stringify(frontRule),
                    enterpriseData,
                    officeList: JSON.stringify(cldOfficeConst.list),
                    status: ctx.service.enterprise.displayStatus,
                    statusClass: ctx.service.enterprise.statusClass,
                    managerList: JSON.stringify(managerList),
                    breadCrumb,
                };

                await this.layout('enterprise/modify.ejs', renderData);
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 保存企业操作
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async save(ctx) {
            let id = ctx.params.id;
            id = parseInt(id);

            try {
                if (isNaN(id) || id < 0) {
                    throw '参数错误';
                }
                // 获取数据规则
                const scene = id === 0 ? 'enterprise' : 'enterpriseModify';
                const rule = ctx.service.enterprise.rule(scene);
                ctx.validate(rule);

                const result = await ctx.service.enterprise.save(ctx.request.body, id);

                if (!result) {
                    throw '保存企业数据失败';
                }

                this.setMessage('保存企业数据成功', this.messageType.SUCCESS);
                ctx.redirect('/enterprise');
            } catch (error) {
                console.log(error);
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 删除企业
         *
         * @param {Object} ctx -egg全局变量
         * @return {void}
         */
        async delete(ctx) {
            let id = ctx.params.id;
            id = parseInt(id);

            try {
                if (isNaN(id) || id <= 0) {
                    throw '参数错误';
                }
                const result = ctx.service.enterprise.deleteProjectById(id);

                if (!result) {
                    throw '删除企业失败';
                }

                this.setMessage('删除企业成功', this.messageType.SUCCESS);
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
            }

            ctx.redirect(ctx.request.header.referer);
        }

        /**
         * 企业设置页面
         *
         * @param {Object} ctx -egg 全局变量
         * @return {void}
         */
        async setting(ctx) {
            // 获取数据规则
            const rule = ctx.service.enterprise.rule('enterpriseSetting');
            const frontRule = ctx.helper.validateConvert(rule);

            const enterpriseData = ctx.session.sessionEnterpriseData;

            try {
                // 查找数据
                if (enterpriseData.id === undefined) {
                    throw '不存在对应的企业数据';
                }

                // 面包屑
                const breadCrumb = this._getBreadCrumb(enterpriseData);

                const renderData = {
                    rule: JSON.stringify(frontRule),
                    enterpriseData,
                    status: ctx.service.enterprise.displayStatus,
                    statusClass: ctx.service.enterprise.statusClass,
                    breadCrumb,
                };
                await this.layout('enterprise/setting.ejs', renderData);
            } catch (error) {
                console.log(error);
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 保存企业设置操作
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        async saveSetting(ctx) {
            let id = ctx.params.id;
            id = parseInt(id);

            try {
                if (isNaN(id) || id < 0) {
                    throw '参数错误';
                }
                // 获取数据规则
                const rule = ctx.service.enterprise.rule('enterpriseSetting');
                ctx.validate(rule);

                const result = await ctx.service.enterprise.saveSetting(ctx.request.body, id);

                if (!result) {
                    throw '保存企业数据失败';
                }

                this.setMessage('保存企业数据成功', this.messageType.SUCCESS);
                ctx.redirect('/enterprise');
            } catch (error) {
                console.log(error);
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 企业项目页面
         *
         * @param {Object} ctx -egg 全局变量
         * @return {void}
         */
        async project(ctx) {
            const enterpriseData = ctx.session.sessionEnterpriseData;

            try {
                // 查找数据
                if (enterpriseData.id === undefined) {
                    throw '不存在对应的企业数据';
                }

                let projectList = await ctx.service.project.getList();
                if (projectList !== null) {
                    let i = 0;
                    for (let project of projectList) {
                        const projectAccountData = await ctx.service.projectAccount.getDataByCondition({ id: project.user_id });
                        projectList[i].username = projectAccountData.name;
                        const tender_num = await ctx.service.tender.count({ project_id: project.id });
                        projectList[i].tender_num = tender_num;
                        i++;
                    }
                }

                // 面包屑
                const breadCrumb = this._getBreadCrumb(enterpriseData);

                const renderData = {
                    enterpriseData,
                    status: ctx.service.enterprise.displayStatus,
                    statusClass: ctx.service.enterprise.statusClass,
                    projectList,
                    breadCrumb,
                };
                await this.layout('enterprise/project.ejs', renderData);
            } catch (error) {
                console.log(error);
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 获取共用面包屑
         *
         * @param {Object} projectData - 企业数据
         * @return {String} - 返回面包屑
         */
        _getBreadCrumb(enterpriseData) {
            // 面包屑
            console.log(this.ctx.topPermission);
            const breadCrumb = [
                { name: this.ctx.topPermission.name },
                { name: '企业版客户', url: this.ctx.controllerName },
                { name: enterpriseData.name },
            ];
            return this.ctx.helper.convertBreadCrumb(breadCrumb, 'replace');
        }

    }

    return EnterpriseController;
};
