'use strict';

/**
 * 后台管理用户控制器
 *
 * @author CaiAoLin
 * @date 2017/9/27
 * @version
 */

const cldOfficeConst = require('../const/cld_office');
module.exports = app => {

    class ManagerController extends app.BaseController {

        /**
         * 用户列表页
         *
         * @param {Object} ctx - egg全局context
         * @return {void}
         */
        async index(ctx) {
            const page = ctx.page;
            const managerSession = ctx.session.managerSession;
            ctx.service.manager.searchFilter(ctx.request.query, managerSession);
            const managerList = await ctx.service.manager.getListWithBuilder();
            const total = await ctx.service.manager.count();
            
            // 判断用户权限和所属办事处（PS:只有总部非造价研究中心和超级管理员可查看所有管理员用户，其它只显示该分组用户）
            let officeList = [];
            const permission = managerSession.permission;
            if (permission === 'all' || parseInt(managerSession.office) === 12) {
                officeList = JSON.stringify(cldOfficeConst.list);
            } else {
                const office = [];
                office[managerSession.office] = cldOfficeConst.list[managerSession.office];
                officeList = JSON.stringify(office);
            }

            // 获取用户组数据
            const groupData = await ctx.service.group.getAllData();
            const groupList = {};
            if (groupData !== null) {
                // 整理数据
                for (const group of groupData) {
                    groupList[group.id] = group.name;
                }
            }

            // 分页相关
            const pageInfo = {
                page,
                total: Math.ceil(total / app.config.pageSize),
                queryData: JSON.stringify(ctx.urlInfo.query),
            };

            // const dropDownMenu = [
            //     { name: '新增管理员', url: ctx.controllerName + '/add/0' },
            // ];

            const renderData = {
                managerList,
                pageInfo,
                groupList,
                officeType: officeList,
                // dropDownMenu,
            };
            await this.layout('manager/index.ejs', renderData);
        }

        /**
         * 新增管理员页面
         *
         * @param {Object} ctx - egg全局context
         * @return {void}
         */
        async add(ctx) {
            // 获取验证规则
            const rule = ctx.service.manager.rule('add');
            const jsValidator = await this.jsValidator.convert(rule).build();

            // 获取用户组数据
            const groupData = await ctx.service.group.getAllData();

            let breadCrumb = [
                { name: '管理员列表', url: ctx.controllerName },
            ];
            breadCrumb = ctx.helper.convertBreadCrumb(breadCrumb);
            const renderData = {
                managerData: {},
                groupData,
                jsValidator,
                breadCrumb,
            };
            await this.layout('manager/save.ejs', renderData);
        }

        /**
         * 编辑用户页面
         *
         * @param {Object} ctx - egg全局context
         * @return {void}
         */
        async modify(ctx) {
            try {
                let id = ctx.params.id;
                id = parseInt(id);
                if (id === 1) {
                    throw '超级管理员不能被修改';
                }
                const rule = ctx.service.manager.rule('modify');
                const jsValidator = await this.jsValidator.convert(rule).build();
                const managerData = await ctx.service.manager.getDataById(id);
                if (managerData === null) {
                    throw '不存在对应管理员';
                }

                // 获取用户组数据
                const groupData = await ctx.service.group.getAllData(false, true);

                let breadCrumb = [
                    { name: '管理员列表', url: ctx.controllerName },
                ];
                breadCrumb = ctx.helper.convertBreadCrumb(breadCrumb);
                const renderData = {
                    jsValidator,
                    groupData: JSON.stringify(groupData),
                    managerData,
                    breadCrumb,
                };
                await this.layout('manager/save.ejs', renderData);

            } catch (error) {
                console.log(error);
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 保存用户数据
         *
         * @param {Object} ctx - egg全局context
         * @return {void}
         */
        async save(ctx) {
            try {
                const id = ctx.params.id;
                // 获取验证规则
                const scene = id > 0 ? 'modify' : 'add';
                const rule = ctx.service.manager.rule(scene);
                ctx.validate(rule);

                const result = await ctx.service.manager.save(ctx.request.body, id);
                if (!result) {
                    throw '保存失败';
                }

                this.setMessage('修改用户数据成功', this.messageType.SUCCESS);
                ctx.redirect('/manager');
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 删除用户
         *
         * @param {Object} ctx - egg全局context
         * @return {Object} - 成功则跳转页面
         */
        async delete(ctx) {
            let id = ctx.params.id;
            id = parseInt(id);
            try {
                if (isNaN(id) || id <= 0) {
                    throw 'id错误';
                }
                const result = await ctx.service.manager.deleteById(id);
                if (!result) {
                    throw '删除失败';
                }

                this.setMessage('删除成功', this.messageType.SUCCESS);
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
            }

            return ctx.redirect(ctx.request.header.referer);
        }

        /**
         * 管理员个人信息修改页面
         *
         * @param {Object} ctx - egg全局context
         * @return {void}
         */
        async profile(ctx) {
            try {
                const sessionManagerData = ctx.session.managerSession;
                const managerData = await ctx.service.manager.getDataById(sessionManagerData.userID);

                if (managerData === null) {
                    throw '不存在对应用户';
                }
                const rule = ctx.service.manager.rule('profile');
                const jsValidator = await this.jsValidator.convert(rule).build();

                const renderData = {
                    managerData,
                    jsValidator,
                };

                await this.layout('manager/profile.ejs', renderData);
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
                ctx.redirect(ctx.request.header.referer);
            }

        }

        /**
         * 保存管理员个人信息
         *
         * @param {Object} ctx - egg全局context
         * @return {Object} - 保存后跳转页面
         */
        async saveProfile(ctx) {
            try {
                const sessionManagerData = ctx.session.managerSession;
                const id = sessionManagerData.userID;
                const postData = ctx.request.body;
                // 非本地用户则加上用于验证的password字段用于通过验证
                if (id !== 1) {
                    postData.password = '';
                    postData.new_password = '';
                    postData.confirm_password = '';
                }

                // 获取验证规则
                const rule = ctx.service.manager.rule('profile');
                ctx.validate(rule);

                const result = await ctx.service.manager.save(postData, id);
                if (!result) {
                    throw '保存失败';
                }

                this.setMessage('保存个人信息成功', this.messageType.SUCCESS);
            } catch (error) {
                console.log(error);
                this.setMessage(error.toString(), this.messageType.ERROR);
            }
            return ctx.redirect(ctx.request.header.referer);
        }

    }

    return ManagerController;
};
