'use strict';

/**
 * 权限相关控制器
 *
 * @author CaiAoLin
 * @date 2017/9/27
 * @version
 */

const maintainConst = require('../const/maintain');
module.exports = app => {

    class PermissionController extends app.BaseController {

        /**
         * 权限列表
         *
         * @param {Object} ctx - egg全局context
         * @return {void}
         */
        async index(ctx) {
            // 获取整理好的列表数据
            const sortedPermission = await ctx.service.permission.getMenu();
            const renderData = {
                permissionList: sortedPermission,
            };
            await this.layout('permission/index.ejs', renderData);
        }

        /**
         * 新增权限页面
         *
         * @param {Object} ctx - egg全局context
         * @return {void}
         */
        async add(ctx) {
            const pid = ctx.params.id;
            const rule = ctx.service.permission.rule();
            const jsValidator = await this.jsValidator.convert(rule).build();

            let breadCrumb = [
                { name: '权限列表', url: ctx.controllerName },
            ];
            breadCrumb = ctx.helper.convertBreadCrumb(breadCrumb);
            const renderData = {
                pid,
                jsValidator,
                permissionData: {},
                breadCrumb,
            };
            await this.layout('permission/save.ejs', renderData);
        }

        /**
         * 编辑权限页面
         *
         * @param {Object} ctx - egg全局context
         * @return {void|Object} 不存在数据则跳转
         */
        async modify(ctx) {
            try {
                const id = ctx.params.id;
                const rule = ctx.service.permission.rule();
                const jsValidator = await this.jsValidator.convert(rule).build();
                const permissionData = await ctx.service.permission.getDataById(id);

                if (permissionData === null) {
                    throw '不存在对应数据';
                }

                let breadCrumb = [
                    { name: '权限列表', url: ctx.controllerName },
                ];
                breadCrumb = ctx.helper.convertBreadCrumb(breadCrumb);
                const renderData = {
                    pid: permissionData.pid,
                    jsValidator,
                    permissionData,
                    breadCrumb,
                };
                await this.layout('permission/save.ejs', renderData);
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
                return ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 保存新权限
         *
         * @param {Object} ctx - egg全局context
         * @return {Object} - 保存后跳转页面
         */
        async save(ctx) {
            try {
                let id = ctx.params.id;
                id = parseInt(id);

                // 获取校验规则
                const rule = ctx.service.permission.rule('add');
                ctx.validate(rule);

                const result = await ctx.service.permission.save(ctx.request.body, id);
                if (!result) {
                    throw '保存失败';
                }

                ctx.redirect('/permission');
            } catch (error) {
                console.log(error);
                this.setMessage(error.toString(), this.messageType.ERROR);
                return ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 删除权限
         *
         * @param {Object} ctx - egg全局context
         * @return {Object} - 删除后跳转页面
         */
        async delete(ctx) {
            let id = ctx.params.id;
            id = parseInt(id);
            try {
                if (isNaN(id) || id <= 0) {
                    throw 'id错误';
                }
                const result = await ctx.service.permission.deleteById(id);
                if (!result) {
                    throw '删除失败';
                }

                // 更新缓存
                ctx.service.permission.flushCache();
                this.setMessage('删除成功', this.messageType.SUCCESS);
            } catch (error) {
                this.setMessage(error.toString(), this.messageType.ERROR);
            }

            return ctx.redirect(ctx.request.header.referer);
        }

        /**
         * 维护设置
         *
         * @param {Object} ctx - egg全局context
         * @return {Object} - 删除后跳转页面
         */
        async maintain(ctx) {
            const maintainData = await ctx.service.maintain.getDataById(1);
            const rule = ctx.service.maintain.rule();
            const jsValidator = await this.jsValidator.convert(rule).build();
            const renderData = {
                maintainData,
                maintainConst,
                jsValidator,
            };
            await this.layout('permission/maintain.ejs', renderData);
        }

        /**
         * 保存维护设置
         *
         * @param {Object} ctx - egg全局context
         * @return {Object} - 保存后跳转页面
         */
        async maintainSet(ctx) {
            try {
                // 获取校验规则
                const rule = ctx.service.maintain.rule();
                ctx.validate(rule);

                const result = await ctx.service.maintain.save(ctx.request.body);
                if (!result) {
                    throw '保存失败';
                }

                ctx.redirect('/maintain');
            } catch (error) {
                console.log(error);
                this.setMessage(error.toString(), this.messageType.ERROR);
                return ctx.redirect(ctx.request.header.referer);
            }
        }

        /**
         * 停止维护设置
         *
         * @param {Object} ctx - egg全局context
         * @return {Object} - 保存后跳转页面
         */
        async maintainStop(ctx) {
            try {
                const result = await ctx.service.maintain.stop();
                if (!result) {
                    throw '保存失败';
                }

                ctx.redirect('/maintain');
            } catch (error) {
                console.log(error);
                this.setMessage(error.toString(), this.messageType.ERROR);
                return ctx.redirect(ctx.request.header.referer);
            }
        }

    }

    return PermissionController;
};
