'use strict';

/**
 * 标准库控制器基类
 *
 * @author CaiAoLin
 * @date 2018/1/10
 * @version
 */

const BaseController = require('../base/base_controller');
const streamToArray = require('stream-to-array');
const sendToWormhole = require('stream-wormhole');
const fs = require('fs');
const path = require('path');
// excel解析
const excel = require('node-xlsx');
const standardConst = require('../const/standard');

class StandardLibController extends BaseController {

    /**
     * 构造函数
     *
     * @param {Object} ctx - egg全局变量
     * @param {Object} setting
     * @return {void}
     */
    constructor(ctx, setting) {
        super(ctx);
        this.model = setting.model;
        this.listModel = setting.listModel;
        this.title = setting.title;
        this.viewFolder = setting.viewFolder;
        this.app = null;
    }

    /**
     * 项目节数据上传
     *
     * @param {Object} ctx - egg全局变量
     * @return {void}
     */
    async upload(ctx) {
        const stream = await ctx.getFileStream();
        let result = true;
        let buffer = null;
        const fileName = path.join(this.app.config.filePath, 'cache', stream.filename);
        const listId = parseInt(stream.fields.id);
        try {
            if (isNaN(listId) || listId <= 0) {
                throw '参数错误';
            }
            // 读取字节流
            const parts = await streamToArray(stream);
            // 转化为buffer
            buffer = Buffer.concat(parts);
            // 写入文件
            await fs.writeFileSync(fileName, buffer);

            // 读取文件
            const sheet = excel.parse(fileName);
            if (sheet[0] === undefined || sheet[0].data === undefined) {
                throw 'excel没有对应数据';
            }

            result = await this.model.importData(sheet[0].data, listId);
            if (!result) {
                throw '导入数据失败!';
            }

            ctx.redirect('/' + ctx.controllerName + '/detail/' + listId);
        } catch (error) {
            this.ctx.helper.log(error);
            // 失败需要消耗掉stream 以防卡死
            await sendToWormhole(stream);
            this.setMessage(error.toString(), this.messageType.ERROR);
            ctx.redirect(ctx.request.header.referrer);
        }
    }

    async uploadJSON(ctx) {
        const stream = await ctx.getFileStream();
        let result = true;
        const fileName = path.join(this.app.config.filePath, 'cache', stream.filename);
        const listId = parseInt(stream.fields.id);
        try {
            if (isNaN(listId) || listId <= 0) {
                throw '参数错误';
            }
            await this.ctx.helper.saveStreamFile(stream, fileName);

            // 读取文件
            const stdData = await fs.readFileSync(fileName, 'utf8');

            await this.model.importJsonData(stdData, listId);

            ctx.redirect('/' + ctx.controllerName + '/detail/' + listId);
        } catch (error) {
            ctx.helper.log(error);
            // 失败需要消耗掉stream 以防卡死
            await sendToWormhole(stream);
            this.setMessage(error.toString(), this.messageType.ERROR);
            ctx.redirect(ctx.request.header.referer);
        }
    }

    /**
     * 根据id获取子项
     *
     * @param {Object} ctx - egg全局变量
     * @return {void}
     */
    async getChildren(ctx) {
        const responseData = {
            err: 0,
            msg: '',
            data: [],
        };
        try {
            let id = ctx.request.body.id;
            let listId = ctx.request.body.listId;
            id = parseInt(id);
            listId = parseInt(listId);
            if (isNaN(id) || id <= 0 || isNaN(listId) || listId <= 0) {
                throw '参数错误';
            }
            const condition = { pid: id, list_id: listId };
            const libData = await this.model.getAllDataByCondition({ where: condition });

            responseData.data = libData;
        } catch (error) {
            this.ctx.helper.log(error);
            responseData.err = 1;
            responseData.msg = error;
        }

        ctx.body = responseData;
    }

    /**
     * 导出数据
     *
     * @param {Object} ctx - egg全局变量
     * @return {void}
     */
    async detailExport(ctx) {
        try {
            let id = ctx.params.id;
            id = parseInt(id);

            if (isNaN(id) || id <= 0) {
                throw '参数错误';
            }

            // 获取导出数据
            const exportData = await this.model.getExportData(id);

            // 存入文件
            const buffer = excel.build([{ name: 'sheet', data: exportData }]);
            const fileName = this.model.tableName + '.xlsx';
            const filePath = path.join(this.app.config.filePath, 'cache', fileName);
            fs.writeFileSync(filePath, buffer, 'binary');
            const stats = fs.statSync(filePath);
            // 下载相关header
            ctx.set({
                'Content-Type': 'application/octet-stream',
                'Content-Disposition': 'attachment; filename=' + fileName,
                'Content-Length': stats.size,
            });
            const stream = fs.createReadStream(filePath).pipe(ctx.res);
            ctx.body = stream.pipe(ctx.res);

            // 输出文件后删除
            fs.unlinkSync(filePath);
        } catch (error) {
            this.ctx.helper.log(error);
            this.setMessage(error.toString(), this.messageType.ERROR);
            ctx.redirect(ctx.request.header.referrer);
        }
    }

    async loadDetailData(ctx) {
        try {
            const listId = ctx.params.id;
            if (isNaN(listId) || listId <= 0) {
                throw '参数错误';
            }
            const data = JSON.parse(ctx.request.body.data);
            if (!data.dType) {
                throw '参数错误';
            }
            if (data.dType === 'all') {
                const libData = await this.model.getData(listId);
                ctx.body = {err: 0, msg: '', data: libData};
            }
        } catch (error) {
            this.ctx.helper.log(error);
            if (error.stack) {
                ctx.body = {err: 1, msg: '未知错误', data: null};
            } else {
                ctx.body = {err: 1, msg: error, data: null};
            }
        }
    }

    /**
     * 列表页
     *
     * @param {Object} ctx - egg全局变量
     * @return {void}
     */
    async index(ctx) {
        const page = ctx.page;

        // 获取标准清单数据
        this.listModel.searchFilter(ctx.request.query);
        const standardList = await this.listModel.getListWithBuilder();
        const total = await this.listModel.count();
        // 获取所有计价规范列表
        const valuationList = await ctx.service.valuationList.getAllDataByCondition();
        let check_array = [];
        switch (this.title) {
            case '工程量清单':
                for (const value of valuationList) {
                    if (value.bill_id !== '' && value.bill_id !== null) {
                        check_array = check_array.concat(value.bill_id.split(','));
                    }
                    if (value.list_bill_id !== '' && value.list_bill_id !== null) {
                        check_array = check_array.concat(value.list_bill_id.split(','));
                    }
                }
                break;
            case '项目节':
                for (const value of valuationList) {
                    if (value.chapter_id !== '' && value.chapter_id !== null) {
                        check_array = check_array.concat(value.chapter_id.split(','));
                    }
                    if (value.list_chapter_id !== '' && value.list_chapter_id !== null) {
                        check_array = check_array.concat(value.list_chapter_id.split(','));
                    }
                }
                break;
            case '台账模板':
                for (const value of valuationList) {
                    if (value.template_id !== '' && value.template_id !== null) {
                        check_array = check_array.concat(value.template_id.split(','));
                    }
                    if (value.list_template_id !== '' && value.list_template_id !== null) {
                        check_array = check_array.concat(value.list_template_id.split(','));
                    }
                }
                break;
            default: break;
        }
        for (const index in standardList) {
            standardList[index].canDel = check_array.indexOf(standardList[index].id.toString()) === -1;
        }

        // 分页相关
        const pageInfo = {
            page,
            total: Math.ceil(total / ctx.app.config.pageSize),
            queryData: JSON.stringify(ctx.urlInfo.query),
        };
        const dropDownMenu = [
            { name: '新增' + this.title, url: ctx.controllerName + '/add/0' },
        ];

        const renderData = {
            standardList,
            pageInfo,
            dropDownMenu,
        };
        await this.layout(this.viewFolder + '/index.ejs', renderData);
    }

    /**
     * 新增
     *
     * @param {Object} ctx - egg全局变量
     * @return {void}
     */
    async add(ctx) {
        const rule = this.listModel.rule();
        const jsValidator = await this.jsValidator.convert(rule).build();

        const standardListData = {};
        const renderData = {
            jsValidator,
            standardListData,
        };
        await this.layout(this.viewFolder + '/save.ejs', renderData);
    }

    /**
     * 保存列表数据
     *
     * @param {Object} ctx - egg全局变量
     * @return {void}
     */
    async save(ctx) {
        try {
            const id = ctx.params.id;
            const rule = this.listModel.rule();
            ctx.validate(rule);

            const result = await this.listModel.save(ctx.request.body, id);
            if (!result) {
                throw '保存失败';
            }

            ctx.redirect('/' + ctx.controllerName);
        } catch (error) {
            this.ctx.helper.log(error);
            this.setMessage(error.toString(), this.messageType.ERROR);
            ctx.redirect(ctx.request.header.referer);
        }
    }

    /**
     * 删除
     *
     * @param {Object} ctx - egg全局变量
     * @return {void}
     */
    async delete(ctx) {
        let id = ctx.params.id;
        id = parseInt(id);
        try {
            if (isNaN(id) || id <= 0) {
                throw 'id错误';
            }
            const result = await this.listModel.delete(id);
            if (!result) {
                throw '删除失败';
            }

            this.setMessage('删除成功', this.messageType.SUCCESS);
        } catch (error) {
            this.setMessage(error.toString(), this.messageType.ERROR);
        }

        ctx.redirect(ctx.request.header.referer);
    }

    /**
     * 详情页面
     *
     * @param {Object} ctx - egg全局变量
     * @return {void}
     */
    async detail(ctx) {
        let id = ctx.params.id;
        try {
            id = parseInt(id);
            if (isNaN(id) || id <= 0) {
                throw '参数错误';
            }

            const dropDownMenu = [
                { name: '导入json数据', url: ctx.controllerName + '/detail-upload/' + id },
            ];

            const renderData ={
                dropDownMenu,
                listId: id,
                nodeType: JSON.stringify(standardConst.nodeType),
                downloadJsonUrl: '/' + ctx.controllerName + '/detail/' + id + '/download/' + this.title + '.json'
            };
            await this.layout(this.viewFolder + '/detail.ejs', renderData);

        } catch (error) {
            this.ctx.helper.log(error);
            ctx.redirect('/' + ctx.controllerName);
        }
    }

    /**
     * 上传页面
     *
     * @param {Object} ctx - egg全局变量
     * @return {void}
     */
    async detailUpload(ctx) {
        const id = ctx.params.id;
        try {
            if (isNaN(id) || id <= 0) {
                throw '参数错误';
            }

            const renderData = {
                id,
            };
            await this.layout(this.viewFolder + '/detail_save.ejs', renderData);
        } catch (error) {
            this.ctx.helper.log(error);
            ctx.redirect(ctx.request.header.referrer);
        }
    }

    async updateDetailData(ctx) {
        try {
            const listId = parseInt(ctx.params.id);
            if (isNaN(listId) || listId <= 0) throw '参数错误';
            const data = JSON.parse(ctx.request.body.data);

            const responseData = {err: 0, msg: '', data: null};
            switch (data.postType) {
                case 'add':
                    responseData.data = await this.model.addNode(listId, data.updateData);
                    break;
                case 'delete':
                    responseData.data = await this.model.delete(listId, data.updateData, data.count);
                    break;
                case 'up-move':
                    responseData.data = await this.model.upMoveNode(listId, data.updateData, data.count);
                    break;
                case 'down-move':
                    responseData.data = await this.model.downMoveNode(listId, data.updateData, data.count);
                    break;
                case 'up-level':
                    responseData.data = await this.model.upLevelNode(listId, data.updateData, data.count);
                    break;
                case 'down-level':
                    responseData.data = await this.model.downLevelNode(listId, data.updateData, data.count);
                    break;
                case 'update':
                    responseData.data = await this.model.updateInfos(listId, data.updateData);
                    break;
                default:
                    throw '未知操作';
            }
            ctx.body = responseData;
        } catch(err) {
            this.ctx.helper.log(err);
            if (err.stack) {
                ctx.body = {err: 1, msg: '未知错误', data: null};
            } else {
                ctx.body = {err: 1, msg: err, data: null};
            }
        }
    }

    async exportDetailJsonData(ctx) {
        const id = ctx.params.id;
        const file = ctx.params.file;
        if (file === this.title + '.json') {
            try {
                const create_time = Date.parse(new Date()) / 1000;
                const fileName = path.join(this.app.config.filePath, 'cache', this.title + '-' + create_time + '.json');
                const data = await this.model.exportJsonData(id);
                await ctx.helper.saveBufferFile(JSON.stringify(data,"","\t"), fileName);
                ctx.body = await fs.readFileSync(fileName);
                // 输出文件后删除
                fs.unlinkSync(fileName);
            } catch(err) {
                this.log(err);
            }
        }
    }
}

module.exports = StandardLibController;
