'use strict';

/**
 * 辅助方法扩展
 *
 * @author CaiAoLin
 * @date 2017/9/28
 * @version
 */

const fs = require('fs');
const path = require('path');
const streamToArray = require('stream-to-array');

module.exports = {
    /**
     * 内置验证器转换为前端jquery验证器
     *
     * @param {Object} rule - 后端service中的rule
     * @return {Object} - 返回转换后的数据
     */
    validateConvert(rule) {
        const result = {};
        if (Object.keys(rule).length <= 0) {
            return rule;
        }

        for (const index in rule) {
            result[index] = {};
            const type = rule[index].type !== undefined && rule[index].type !== '' ? rule[index].type : '';
            const stringType = ['string', 'password'];
            // 是否必填
            if (rule[index].required !== undefined) {
                result[index].required = rule[index].required;
            }

            // 最小长度
            if (stringType.indexOf(type) >= 0 && rule[index].min !== undefined) {
                result[index].minlength = rule[index].min;
            }

            // 最大长度
            if (stringType.indexOf(type) >= 0 && rule[index].max !== undefined) {
                result[index].maxlength = rule[index].max;
            }

            // 密码相关
            if (type === 'password' && rule[index].compare !== undefined) {
                result[index].equalTo = '#' + rule[index].compare;
            }

            // 最小值
            const integerType = ['integer', 'int', 'Number'];
            if (integerType.indexOf(type) >= 0 && rule[index].min !== undefined) {
                result[index].min = rule[index].min;
            }

            // 最大值
            if (integerType.indexOf(type) >= 0 && rule[index].max !== undefined) {
                result[index].max = rule[index].max;
            }

            // 自定义判断
            const customType = ['mobile', 'ip'];
            // 自定义且带参数
            if (customType.indexOf(type) >= 0) {
                result[index][type] = true;
                if (rule[index].allowEmpty !== undefined) {
                    result[index].required = !rule[index].allowEmpty;
                }
            }

        }

        return result;
    },

    generateProjectCodeString(length, minNumberLength) {
        const stringSeed = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'P', 'Q', 'R', 'S',
            'T', 'U', 'V', 'W', 'X', 'Y', 'Z'];
        const numberSeed = [2, 3, 4, 5, 6, 7, 8, 9];
        let match = false, result;
        while (!match) {
            result = '';

            for (let i = 0; i < length; i++) {
                let seed;
                if (i === 0) {
                    seed = stringSeed;
                } else {
                    seed = stringSeed.concat(numberSeed);
                }
                const seedLength = seed.length - 1;
                const index = Math.ceil(Math.random() * seedLength) - 1;

                result += seed[index];
            }
            const num = result.match(/[2-9]/g);
            match = num && num.length >= minNumberLength;
        }

        return result;
    },

    /**
     * 生成随机字符串
     *
     * @param {Number} length - 需要生成字符串的长度
     * @param {Number} type - 1为数字和字符 2为纯数字 3为纯字母
     * @return {String} - 返回生成结果
     */
    generateRandomString(length, type = 1) {
        length = parseInt(length);
        length = isNaN(length) ? 1 : length;
        let randSeed = [];
        let numberSeed = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9];
        let stringSeed = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S',
            'T', 'U', 'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o',
            'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z'];

        switch (type) {
            case 1:
                randSeed = stringSeed.concat(numberSeed);
                stringSeed = numberSeed = null;
                break;
            case 2:
                randSeed = numberSeed;
                break;
            case 3:
                randSeed = stringSeed;
                break;
            default:
                break;
        }

        const seedLength = randSeed.length - 1;
        let result = '';
        for (let i = 0; i < length; i++) {
            const index = Math.ceil(Math.random() * seedLength);
            result += randSeed[index];
        }

        return result;
    },

    /**
     * 显示排序符号
     *
     * @param {String} field - 字段名称
     * @return {String} - 返回字段排序的符号
     */
    showSortFlag(field) {
        const sort = this.ctx.sort;
        if (!(sort instanceof Array) || sort.length !== 2) {
            return '';
        }
        sort[1] = sort[1].toUpperCase();
        return (sort[0] === field && sort[1] === 'DESC') ? '' : '-';
    },

    /**
     * 判断是否为ajax请求
     *
     * @param {Object} request - 请求数据
     * @return {boolean} 判断结果
     */
    isAjax(request) {
        let headerInfo = request.headers['x-requested-with'] === undefined ? '' : request.headers['x-requested-with'];
        headerInfo = headerInfo.toLowerCase();
        return headerInfo === 'xmlhttprequest';
    },

    /**
     * 模拟发送请求
     *
     * @param {String} url - 请求地址
     * @param {Object} data - 请求数据
     * @param {String} type - 请求类型(POST) POST | GET
     * @param {String} dataType - 数据类型 json|text
     * @return {Object} - 请求结果
     */
    async sendRequest(url, data, type = 'POST', dataType = 'json') {
        // 发起请求
        const response = await this.ctx.curl(url, {
            method: type,
            data,
            dataType,
        });
        if (response.status !== 200) {
            throw '请求失败';
        }

        return response.data;
    },

    /**
     * 转换面包屑导航数据
     *
     * @param {Object} breadCrumb - 面包屑数据
     * @param {String} mode - 模式 append 为追加型模式 \ replace 为全部替换模式
     * @return {String} - 转换为字符串数据
     */
    convertBreadCrumb(breadCrumb, mode = 'append') {
        let result = '';
        if (!breadCrumb instanceof Array || breadCrumb.length <= 0) {
            return result;
        }
        if (mode === 'append') {
            const tmpBreadCrumb = [];
            tmpBreadCrumb.push({ name: this.ctx.topPermission.name });
            tmpBreadCrumb.push.apply(tmpBreadCrumb, breadCrumb);
            tmpBreadCrumb.push({ name: this.ctx.currentName });
            breadCrumb = tmpBreadCrumb;
        }
        const tmpList = [];
        for (const data of breadCrumb) {
            if (data.name === undefined) {
                continue;
            }
            const tmp = data.url === undefined || data.url === '' ?
                data.name : '<a href="/' + data.url + '">' + data.name + '</a>';
            tmpList.push(tmp);
        }
        result = tmpList.join(' / ');

        return result;
    },

    /**
     * 深度验证数据
     *
     * @param {Object} rule - 数据规则
     * @return {void}
     */
    validate(rule) {
        // 先用内置的验证器验证数据
        this.ctx.validate(rule);

        // 然后再验证是否有多余的数据
        const postData = this.ctx.request.body;
        delete postData._csrf;

        const postDataKey = Object.keys(postData);
        const ruleKey = Object.keys(rule);
        // 自动增加字段则填充上，以防判断出错
        if (postData.create_time !== undefined) {
            ruleKey.push('create_time');
        }
        for (const tmp of postDataKey) {
            // 规则里面没有定义则抛出异常
            if (ruleKey.indexOf(tmp) < 0) {
                throw '参数不正确';
            }
        }
    },

    /**
     * 拆分path
     *
     * @param {String|Array} paths - 拆分字符
     * @param {String} symbol - 拆分符号
     * @return {Array} - 拆分结果
     */
    explodePath(paths, symbol = '-') {
        const result = [];
        paths = paths instanceof Array ? paths : [paths];
        for (const path of paths) {
            // 拆分数据
            const pathArray = path.split(symbol);
            // 用户缓存循环的数据
            const tmpArray = [];
            for (const tmp of pathArray) {
                // 每次循环都追加一个数据进去
                tmpArray.push(tmp);
                const tmpPathString = tmpArray.join(symbol);
                // 判断是否已经存在有对应数据
                if (result.indexOf(tmpPathString) >= 0) {
                    continue;
                }
                result.push(tmpPathString);
            }
        }

        return result;
    },

    /**
     * 递归创建文件夹(fs.mkdirSync需要上一层文件夹已存在)
     * @param pathName
     * @returns {Promise<void>}
     */
    async recursiveMkdirSync(pathName) {
        const upperPath = path.dirname(pathName);
        if (!fs.existsSync(upperPath)) {
            await this.recursiveMkdirSync(upperPath);
        }
        await fs.mkdirSync(pathName);
    },

    /**
     * 字节 保存至 本地文件
     * @param buffer - 字节
     * @param fileName - 文件名
     * @returns {Promise<void>}
     */
    async saveBufferFile(buffer, fileName) {
        // 检查文件夹是否存在，不存在则直接创建文件夹
        const pathName = path.dirname(fileName);
        if (!fs.existsSync(pathName)) {
            await this.recursiveMkdirSync(pathName);
        }
        await fs.writeFileSync(fileName, buffer);
    },

    /**
     * 将文件流的数据保存至本地文件
     * @param stream
     * @param fileName
     * @returns {Promise<void>}
     */
    async saveStreamFile(stream, fileName) {
        // 读取字节流
        const parts = await streamToArray(stream);
        // 转化为buffer
        const buffer = Buffer.concat(parts);
        // 写入文件
        await this.saveBufferFile(buffer, fileName);
    },

    log(error) {
        if (error.stack) {
            this.ctx.logger.error(error);
        } else {
            this.ctx.getLogger('fail').info(JSON.stringify({
                error: error,
                project: this.ctx.session.sessionProject,
                user: this.ctx.session.sessionUser,
                body: this.ctx.session.body,
            }));
        }
    },

    ajaxErrorBody(error, msg) {
        return error.statck 
            ? {err: 2, msg: msg, data: null} 
            : {err: 1, msg: error.toString(), data: null};
    },

    /**
     * 比较编码
     * @param str1
     * @param str2
     * @param symbol
     * @returns {number}
     */
    compareCode(str1, str2, symbol = '-') {
        if (!str1) {
            return -1;
        } else if (!str2) {
            return 1;
        }

        const path1 = str1.split(symbol);
        const path2 = str2.split(symbol);
        const reg = /^[0-9]*$/;
        for (let i = 0, iLen = Math.min(path1.length, path2.length); i < iLen; i++) {
            if (reg.test(path1[i]) && reg.test(path2[i])) {
                const num1 = parseInt(path1[i]);
                const num2 = parseInt(path2[i]);
                if (num1 !== num2)  {
                    return num1 - num2;
                }
            } else if (path1[i] < path2[i]) {
                return -1;
            } else if (path1[i] > path2[i]) {
                return 1;
            }
        }
        return path1.length - path2.length;
    },


};
