/**
 *
 * 构建pathTree
 * 可动态加载子节点，要求子节点获取接口按/xxx/get-children定义
 * @param {Object} setting - 设置
 * @returns {PathTree}
 */

'use strict';

const itemsPre = 'id_';

const createNewPathTree = function (type, setting) {
    class BaseTree {
        /**
         * 构造函数
         */
        constructor (setting) {
            // 无索引
            this.datas = [];
            // 以key为索引
            this.items = {};
            // 以排序为索引
            this.nodes = [];
            // 根节点
            this.children = [];
            // 树设置
            this.setting = setting;
        }
        /**
         * 树结构根据显示排序
         */
        sortTreeNode (isResort) {
            const self = this;
            const addSortNodes = function (nodes) {
                if (!nodes) { return }
                for (let i = 0; i < nodes.length; i++) {
                    self.nodes.push(nodes[i]);
                    nodes[i].index = self.nodes.length - 1;
                    if (!isResort) {
                        nodes[i].children = self.getChildren(nodes[i]);
                    } else {
                        nodes[i].children.sort(function (a, b) {
                            return a.order - b.order;
                        })
                    }
                    addSortNodes(nodes[i].children);
                }
            };
            this.nodes = [];
            if (!isResort) {
                this.children = this.getChildren();
            } else {
                this.children.sort(function (a, b) {
                    return a.order - b.order;
                })
            }
            addSortNodes(this.children);
        }
        /**
         * 加载数据(初始化), 并给数据添加部分树结构必须数据
         * @param datas
         */
        loadDatas (datas) {
            // 清空旧数据
            this.items = {};
            this.nodes = [];
            this.datas = [];
            this.children = [];
            // 加载全部数据
            datas.sort(function (a, b) {
                return a.level - b.level;
            });
            for (const data of datas) {
                const keyName = itemsPre + data[this.setting.id];
                if (!this.items[keyName]) {
                    const item = JSON.parse(JSON.stringify(data));
                    item.children = [];
                    item.expanded = true;
                    item.visible = true;
                    this.items[keyName] = item;
                    this.datas.push(item);
                    if (item[setting.pid] === setting.rootId) {
                        this.children.push(item);
                    } else {
                        const parent = this.getParent(item);
                        if (parent) {
                            parent.children.push(item);
                        }
                    }
                }
            }
            this.children.sort(function (a, b) {
                return a.order - b.order;
            });
            this.sortTreeNode(true);
            if (this.setting.autoExpand >= 0) this.expandByLevel(this.setting.autoExpand);
        }

        getItemsByIndex(index) {
            return this.nodes[index];
        }
        /**
         * 根据id获取树结构节点数据
         * @param {Number} id
         * @returns {Object}
         */
        getItems (id) {
            return this.items[itemsPre + id];
        };
        /**
         * 查找node的parent
         * @param {Object} node
         * @returns {Object}
         */
        getParent (node) {
            return this.getItems(node[this.setting.pid]);
        };
        getAllParents (node) {
            const parents = [];
            if (node.full_path && node.full_path !== '') {
                const parentIds = node.full_path.split('-');
                for (const id of parentIds) {
                    if (id !== node[this.setting.id]) {
                        parents.push(this.getItems(id));
                    }
                }
            } else {
                let vP = this.getParent(node);
                while (vP) {
                    parents.push(vP);
                    vP = this.getParent(vP);
                }
            }
            return parents;
        }

        /**
         * 查找node的前兄弟节点
         * @param node
         * @returns {*}
         */
        getPreSiblingNode(node) {
            if (!node) return null;
            const parent = this.getParent(node);
            const siblings = parent ? parent.children : this.children;
            const index = siblings.indexOf(node);
            return (index > 0) ? siblings[index - 1] : null;
        }
        /**
         * 查找node的后兄弟节点
         * @param node
         * @returns {*}
         */
        getNextSiblingNode(node) {
            const parent = this.getParent(node);
            const siblings = parent ? parent.children : this.children;
            const index = siblings.indexOf(node);
            if (index >= 0 && index < siblings.length - 1) {
                return siblings[index + 1];
            } else {
                return null;
            }
        }

        /**
         * 根据path查找完整节点
         * @param {Number} path
         */
        getFullPathNodes (path) {
            const self = this, ids = path.split('-');
            if (ids.length > 0) {
                return this.nodes.filter((x) => {
                    return ids.indexOf('' + x[self.setting.id]) >= 0;
                });
            } else {
                return [];
            }
        };
        /**
         * 查询node的已下载子节点
         * @param {Object} node
         * @returns {Array}
         */
        getChildren (node) {
            const setting = this.setting;
            const pid = node ? node[setting.id] : setting.rootId;
            const children = this.datas.filter(function (x) {
                return x[setting.pid] === pid;
            });
            children.sort(function (a, b) {
                return a.order - b.order;
            });
            return children;
        };

        /**
         * 递归方式 查询node的已下载的全部后代 (兼容full_path不存在的情况)
         * @param node
         * @returns {*}
         * @private
         */
        _recursiveGetPosterity (node) {
            let posterity = node.children;
            for (const c of node.children) {
                posterity = posterity.concat(this._recursiveGetPosterity(c));
            }
            return posterity;
        };
        /**
         * 查询node的已下载的全部后代
         * @param {Object} node
         * @returns {Array}
         */
        getPosterity (node) {
            if (node.full_path !== '') {
                const reg = new RegExp('^' + node.full_path + '-');
                return this.datas.filter(function (x) {
                    return reg.test(x.full_path);
                });
            } else {
                return this._recursiveGetPosterity(node);
            }
        };
        /**
         * 查询node是否是父节点的最后一个子节点
         * @param {Object} node
         * @returns {boolean}
         */
        isLastSibling (node) {
            const siblings = this.getChildren(this.getParent(node));
            return (siblings && siblings.length > 0) ?  node.order === siblings[siblings.length - 1].order : false;
        };
        /**
         * 刷新子节点是否可见
         * @param {Object} node
         * @private
         */
        _refreshChildrenVisible (node) {
            if (!node.children) {
                node.children = this.getChildren(node);
            }
            if (node.children && node.children.length > 0) {
                for (const child of node.children) {
                    child.visible = node.expanded && node.visible;
                    this._refreshChildrenVisible(child);
                }
            }
        };
        /**
         * 设置节点是否展开, 并控制子节点可见
         * @param {Object} node
         * @param {Boolean} expanded
         */
        setExpanded (node, expanded) {
            node.expanded = expanded;
            this._refreshChildrenVisible(node);
        };

        /**
         * 提取节点key和索引数据
         * @param {Object} node - 节点
         * @returns {key}
         */
        getNodeKeyData (node) {
            const data = {};
            for (const key of this.setting.keys) {
                data[key] = node[key];
            }
            return data;
        };
        /**
         * 得到树结构构成id
         * @param node
         * @returns {*}
         */
        getNodeKey (node) {
            return node[this.setting.id];
        };

        /**
         * 递归 设置节点展开状态
         * @param {Array} nodes - 需要设置状态的节点
         * @param {Object} parent - nodes的父节点
         * @param {Function} checkFun - 判断节点展开状态的方法
         * @private
         */
        _recursiveExpand(nodes, parent, checkFun) {
            for (const node of nodes) {
                node.expanded = checkFun(node);
                node.visible = parent ? (parent.expanded && parent.visible) : true;
                this._recursiveExpand(node.children, node, checkFun);
            }
        }
        /**
         * 自定义展开规则
         * @param checkFun
         */
        expandByCustom(checkFun) {
            this._recursiveExpand(this.children, null, checkFun);
        }
        /**
         * 展开到第几层
         * @param {Number} level - 展开层数
         */
        expandByLevel(level) {
            // function recursiveExpand(nodes, parent) {
            //     for (const node of nodes) {
            //         node.expanded = node.level < level;
            //         node.visible = parent ? (parent.expanded && parent.visible) : true;
            //         recursiveExpand(node.children, node);
            //     }
            // }
            // recursiveExpand(this.children);
            this.expandByCustom(function (n) {
                return n.level < level;
            });
        }

        /**
         * 自动展开节点node
         * @param node
         * @returns {*}
         */
        autoExpandNode(node) {
            const parents = this.getAllParents(node);
            const reload = [];
            for (const p of parents) {
                if (!p.expanded) {
                    reload.push(p);
                    this.setExpanded(p, true);
                }
            }
            return reload;
        }

        /**
         * 加载数据(动态)，只加载不同部分
         * @param {Array} datas
         * @return {Array} 加载到树的数据
         * @privateA
         */
        _updateData (datas) {
            datas = datas instanceof Array ? datas : [datas];
            let loadedData = [];
            for (const data of datas) {
                let node = this.getItems(data[this.setting.id]);
                if (node) {
                    for (const prop in node) {
                        if (data[prop] !== undefined && data[prop] !== node[prop]) {
                            if (prop === this.setting.pid) {
                                loadedData.push(this.getItems(node[this.setting.pid]));
                                loadedData.push(this.getItems(data[this.setting.pid]));
                            }
                            if (prop === this.setting.order) {
                                loadedData = loadedData.concat(this.getPosterity(node));
                            }
                            node[prop] = data[prop];
                        }
                    }
                    loadedData.push(node);
                }
            }
            loadedData = _.uniq(loadedData);
            for (const node of loadedData) {
                if (node) {
                    node.children = this.getChildren(node);
                    node.expanded = node.children.length === 0 ? true : node.children[0].visible;
                } else {
                    this.children = this.getChildren(null);
                }
            }
            this.sortTreeNode(true);
            return loadedData;
        };
        /**
         * 加载数据(动态)，只加载不同部分
         * @param {Array} datas
         * @return {Array} 加载到树的数据
         * @privateA
         */
        _loadData (datas) {
            datas = datas instanceof Array ? datas : [datas];
            const loadedData = [], resortData = [];
            for (const data of datas) {
                let node = this.getItems(data[this.setting.id]);
                if (node) {
                    const parent = this.getItems(node[this.setting.pid]);
                    for (const prop in node) {
                        if (data[prop] !== undefined && data[prop] !== node[prop]) {
                            node[prop] = data[prop];
                            if (parent && resortData.indexOf(parent) === -1) {
                                resortData.push(parent);
                            }
                        }
                    }
                    loadedData.push(node);
                } else {
                    const keyName = itemsPre + data[this.setting.id];
                    const node = JSON.parse(JSON.stringify(data));
                    this.items[keyName] = node;
                    this.datas.push(node);
                    node.expanded = true;
                    node.visible = true;
                    loadedData.push(node);
                    if (resortData.indexOf(node) === -1) {
                        resortData.push(node);
                    }
                    const parent = this.getItems(node[this.setting.pid]);
                    if (parent && resortData.indexOf(parent) === -1) {
                        resortData.push(parent);
                    } else {
                        resortData.push(this.setting.rootId);
                    }
                }
            }
            for (const node of resortData) {
                if (node && node !== this.setting.rootId) {
                    node.children = this.getChildren(node);
                } else {
                    this.children = this.getChildren(null);
                }
            }
            this.sortTreeNode(true);
            for (const node of loadedData) {
                if (!node.expanded) {
                    this.setExpanded(node, true);
                }
            }
            return loadedData;
        };
        /**
         * 清理数据(动态)
         * @param datas
         * @private
         */
        _freeData (datas) {
            datas = datas instanceof Array ? datas : [datas];
            const freeDatas = [];
            const removeArrayData = function (array, data) {
                const index = array.indexOf(data);
                array.splice(index, 1);
            };
            for (const data of datas) {
                const node = this.getItems(data[this.setting.id]);
                if (node) {
                    freeDatas.push(node);
                    node.deleteIndex = this.nodes.indexOf(node);
                    delete this.items[itemsPre + node[this.setting.id]];
                    if (node[this.setting.pid] !== this.setting.rootId) {
                        const parent = this.getItems(node[this.setting.pid]);
                        if (parent) {
                            removeArrayData(parent.children, node);
                        }
                    } else {
                        removeArrayData(this.children, node);
                    }
                    removeArrayData(this.datas, node);
                }
            }
            for(const node of freeDatas) {
                removeArrayData(this.nodes, node);
            }
            return freeDatas;
        };
        /**
         * 加载需展开的数据
         * @param {Array} datas
         * @returns {Array}
         * @private
         */
        _loadExpandData (datas) {
            datas = datas instanceof Array ? datas : [datas];
            const loadedData = [], existData = [], expandData = [], resortData = [];
            for (const data of datas) {
                let node = this.getItems(data[this.setting.id]);
                if (node) {
                    existData.push(node);
                } else {
                    const keyName = itemsPre + data[this.setting.id];
                    const node = JSON.parse(JSON.stringify(data));
                    this.items[keyName] = node;
                    this.datas.push(node);
                    node.expanded = false;
                    node.visible = true;
                    loadedData.push(node);

                    if (resortData.indexOf(node) === -1) {
                        resortData.push(node);
                    }
                    const parent = this.getItems(node[this.setting.pid]);
                    if (parent && resortData.indexOf(parent) === -1) {
                        resortData.push(parent);
                    }
                }
            }
            for (const node of resortData) {
                node.children = this.getChildren(node);
            }
            this.sortTreeNode(true);
            for (const node of loadedData) {
                if (!node.expanded) {
                    this.setExpanded(node, true);
                }
            }
            for (const node of existData) {
                const parent = this.getItems(node[this.setting.pid]);
                if (expandData.indexOf(parent) === -1) {
                    expandData.push(parent);
                    if (!parent.expanded) {
                        this.setExpanded(parent, true);
                    }
                }
                if (!node.expanded) {
                    this.setExpanded(node, true);
                }
            }
            return [loadedData, expandData];
        };

        /**
         * 因为提交其他数据，引起的树结构数据更新，调用该方法
         *
         * @param data - 更新的数据 {update, create, delete}
         * @returns {{}}
         */
        loadPostData(data) {
            const result = {};
            if (data.delete) {
                result.delete = this._freeData(data.delete);
            }
            if (data.create) {
                result.create = this._loadData(data.create);
            }
            if (data.update) {
                result.update = this._updateData(data.update);
            }
            return result;
        }
    }

    class FxTree extends BaseTree {

        /**
         * 检查节点是否是最底层项目节
         * @param node
         * @returns {boolean}
         */
        isLeafXmj(node) {
            if (!node.code) {
                return false;
            }
            for (const child of node.children) {
                if (!child.b_code || child.b_code === '') {
                    return false;
                }
            }
            return true;
        }

        /**
         * 查询最底层项目节(本身或父项)
         * @param {Object} node - 查询节点
         * @returns {Object}
         */
        getLeafXmjParent(node) {
            let parent = node;
            while (parent) {
                if (this.isLeafXmj(parent)) {
                    return parent;
                } else {
                    parent = this.getParent(parent);
                }
            }
            return null;
        }

        /**
         * 展开至最底层项目节
         */
        expandToLeafXmj() {
            const self = this;
            this.expandByCustom(function (node) {
                if (node.b_code && node.b_code !== '') {
                    return false;
                } else {
                    return !self.isLeafXmj(node);
                }
            })
        }

        /**
         * 展开至计算项
         */
        expandByCalcFields() {
            const self = this;
            this.expandByCustom(function (node) {
                for (const field of self.setting.calcFields) {
                    if (node[field]) {
                        return true;
                    }
                }
                return false;
            })
        }
    }

    class LedgerTree extends FxTree {

        /**
         * 加载数据(动态)，只加载不同部分
         * @param {Array} datas
         * @return {Array} 加载到树的数据
         * @privateA
         */
        _updateData (datas) {
            datas = datas instanceof Array ? datas : [datas];
            let loadedData = [];
            for (const data of datas) {
                let node = this.getItems(data[this.setting.id]);
                if (node) {
                    for (const prop in node) {
                        if (data[prop] !== undefined && data[prop] !== node[prop]) {
                            if (prop === this.setting.pid) {
                                loadedData.push(this.getItems(node[this.setting.pid]));
                                loadedData.push(this.getItems(data[this.setting.pid]));
                            }
                            if (prop === this.setting.order) {
                                loadedData = loadedData.concat(this.getPosterity(node));
                            }
                            node[prop] = data[prop];
                        }
                    }
                    loadedData.push(node);
                }
            }
            loadedData = _.uniq(loadedData);
            for (const node of loadedData) {
                node.children = this.getChildren(node);
                node.expanded = node.children.length === 0 ? true : node.children[0].visible;
            }
            this.sortTreeNode(true);
            return loadedData;
        };
        /**
         * 加载数据(动态)，只加载不同部分
         * @param {Array} datas
         * @return {Array} 加载到树的数据
         * @privateA
         */
        _loadData (datas) {
            datas = datas instanceof Array ? datas : [datas];
            const loadedData = [], resortData = [];
            for (const data of datas) {
                let node = this.getItems(data[this.setting.id]);
                if (node) {
                    const parent = this.getItems(node[this.setting.pid]);
                    for (const prop in node) {
                        if (data[prop] !== undefined && data[prop] !== node[prop]) {
                            node[prop] = data[prop];
                            if (parent && resortData.indexOf(parent) === -1) {
                                resortData.push(parent);
                            }
                        }
                    }
                    loadedData.push(node);
                } else {
                    const keyName = itemsPre + data[this.setting.id];
                    const node = JSON.parse(JSON.stringify(data));
                    this.items[keyName] = node;
                    this.datas.push(node);
                    node.expanded = true;
                    node.visible = true;
                    loadedData.push(node);
                    if (resortData.indexOf(node) === -1) {
                        resortData.push(node);
                    }
                    const parent = this.getItems(node[this.setting.pid]);
                    if (parent && resortData.indexOf(parent) === -1) {
                        resortData.push(parent);
                    }
                }
            }
            for (const node of resortData) {
                node.children = this.getChildren(node);
            }
            this.sortTreeNode(true);
            for (const node of loadedData) {
                if (!node.expanded) {
                    this.setExpanded(node, true);
                }
            }
            return loadedData;
        };
        /**
         * 清理数据(动态)
         * @param datas
         * @private
         */
        _freeData (datas) {
            datas = datas instanceof Array ? datas : [datas];
            const freeDatas = [];
            const removeArrayData = function (array, data) {
                const index = array.indexOf(data);
                array.splice(index, 1);
            };
            for (const data of datas) {
                const node = this.getItems(data[this.setting.id]);
                if (node) {
                    freeDatas.push(node);
                    delete this.items[itemsPre + node[this.setting.id]];
                    if (node[this.setting.pid] !== this.setting.rootId) {
                        const parent = this.getItems(node[this.setting.pid]);
                        if (parent) {
                            removeArrayData(parent.children, node);
                        }
                    }
                    removeArrayData(this.datas, node);
                    removeArrayData(this.nodes, node);
                }
            }
            return freeDatas;
        };
        /**
         * 加载需展开的数据
         * @param {Array} datas
         * @returns {Array}
         * @private
         */
        _loadExpandData (datas) {
            datas = datas instanceof Array ? datas : [datas];
            const loadedData = [], existData = [], expandData = [], resortData = [];
            for (const data of datas) {
                let node = this.getItems(data[this.setting.id]);
                if (node) {
                    existData.push(node);
                } else {
                    const keyName = itemsPre + data[this.setting.id];
                    const node = JSON.parse(JSON.stringify(data));
                    this.items[keyName] = node;
                    this.datas.push(node);
                    node.expanded = false;
                    node.visible = true;
                    loadedData.push(node);

                    if (resortData.indexOf(node) === -1) {
                        resortData.push(node);
                    }
                    const parent = this.getItems(node[this.setting.pid]);
                    if (parent && resortData.indexOf(parent) === -1) {
                        resortData.push(parent);
                    }
                }
            }
            for (const node of resortData) {
                node.children = this.getChildren(node);
            }
            this.sortTreeNode(true);
            for (const node of loadedData) {
                if (!node.expanded) {
                    this.setExpanded(node, true);
                }
            }
            for (const node of existData) {
                const parent = this.getItems(node[this.setting.pid]);
                if (expandData.indexOf(parent) === -1) {
                    expandData.push(parent);
                    if (!parent.expanded) {
                        this.setExpanded(parent, true);
                    }
                }
                if (!node.expanded) {
                    this.setExpanded(node, true);
                }
            }
            return [loadedData, expandData];
        };

        /**
         *
         * @param parent
         * @param node
         * @private
         */
        _getNodesParents(parents, nodes) {
            for (const node of nodes) {
                const parent = this.getParent(node);
                if (parent) {
                    const paths = this.getFullPathNodes(parent.full_path);
                    for (const p of paths) {
                        if (parents.indexOf(p) === -1) {
                            parents.push(p);
                        }
                    }
                }
                if (this.getItems(node.ledger_id) && node.children.length > 0) {
                    parents.push(node);
                }
            }
        }

        _getReCalcNodes(reCalcNodes, nodes) {
            for (const node of nodes) {
                const parent = this.getParent(node);
                if (parent) {
                    const paths = this.getFullPathNodes(parent.full_path);
                    for (const p of paths) {
                        if (reCalcNodes.indexOf(p) === -1) {
                            reCalcNodes.push(p);
                        }
                    }
                }
                // 最底层项目节，也需要计算
                //if (this.getItems(node.ledger_id) && node.children.length > 0) {
                    reCalcNodes.push(node);
                //}
            }
        }

        /**
         * 因为提交其他数据，引起的树结构数据更新，调用该方法
         *
         * @param data - 更新的数据 {update, create, delete}
         * @returns {{}}
         */
        loadPostData(data) {
            const result = {}, reCalcNodes = [];
            if (data.update) {
                result.update = this._updateData(data.update);
                this._getReCalcNodes(reCalcNodes, result.update);
            }
            if (data.create) {
                result.create = this._loadData(data.create);
                this._getReCalcNodes(reCalcNodes, result.create);
            }
            if (data.delete) {
                result.delete = this._freeData(data.delete);
                this._getReCalcNodes(reCalcNodes, result.delete);
            }
            reCalcNodes.sort((a, b) => {
                return b.level - a.level;
            });
            for (const node of reCalcNodes) {
                treeCalc.calculateNode(this, node, this.setting.calcFields, this.setting.calcFun);
            }
            result.update = result.update ? result.update.concat(reCalcNodes) : reCalcNodes;
            return result;
        }

        /**
         * 以下方法需等待响应, 通过callback刷新界面
         */
        /**
         * 加载子节点
         * @param {Object} node
         * @param {function} callback
         */
        loadChildren (node, callback) {
            const self = this;
            const url = this.setting.preUrl ? this.setting.preUrl + '/get-children' : 'get-children';
            postData(url, this.getNodeKeyData(node), function (data) {
                self._loadData(data);
                callback();
            });
        };
        /**
         * 树结构基本操作
         * @param {String} url - 请求地址
         * @param {Object} node - 操作节点
         * @param {String} type - 操作类型
         * @param {function} callback - 界面刷新
         */
        baseOperation (url, node, type, callback) {
            const self = this;
            const data = {
                id: node[this.setting.id],
                postType: type
            };
            postData(url, data, function (datas) {
                const refreshData = self.loadPostData(datas);
                callback(refreshData);
            });
        };
        /**
         * 节点数据编辑
         * @param {String} url - 请求地址
         * @param {Array|Object} updateData - 需更新的数据
         * @param {function} callback - 界面刷新
         */
        update (url, updateData, callback) {
            const self = this;
            postData(url, updateData, function (datas) {
                const refreshData = self.loadPostData(datas);
                callback(refreshData);
            }, function () {
                if (updateData instanceof Array) {
                    const result = [];
                    for (const data of updateData) {
                        result.push(self.getItems(data[self.setting.id]));
                    }
                    callback(result)
                } else {
                    callback([self.getItems(updateData[self.setting.id])]);
                }
            });
        };
        /**
         * 复制粘贴整块（目前仅可粘贴为后项）
         * @param {String} url - 请求地址
         * @param {Object} node - 操作节点
         * @param {Array} block - 被复制整块的节点列表
         * @param {function} callback - 界面刷新
         */
        pasteBlock (url, node, block, callback) {
            const self = this;
            const data = {
                id: node[self.setting.id],
                block: block
            };
            postData(url, data, function (datas) {
                const refreshData = self.loadPostData(datas);
                callback(refreshData);
            });
        };
        /**
         * 提交数据
         * @param {String} url - 请求地址
         * @param {Object} node - 当前选中节点
         * @param {Object} data - 提交的数据
         * @param {function} callback - 界面刷新
         */
        postData (url, node, data, callback) {
            const self = this;
            if (node) {
                data.id = node[self.setting.id];
            }
            postData(url, data, function (datas) {
                const refreshData = self.loadPostData(datas);
                callback(refreshData);
                // const result = {};
                // if (datas.update) {
                //     result.update = self._updateData(datas.update);
                // }
                // if (datas.create) {
                //     result.create = self._loadData(datas.create);
                // }
                // if (datas.delete) {
                //     result.delete = self._freeData(datas.delete);
                // }
                // if (datas.expand) {
                //     const [create, update] = self._loadExpandData(datas.expand);
                //     result.create = result.create ? result.create.concat(create) : create;
                //     result.expand = update;
                // }
                // callback(result);
            });
        };
    }

    if (type === 'base') {
        return new BaseTree(setting);
    } else if (type === 'fx') {
        return new FxTree(setting);
    }  else if (type === 'ledger') {
        return new LedgerTree(setting);
    }
};
