'use strict';

/**
 * 客户信息数据模型
 *
 * @author CaiAoLin
 * @date 2017/11/14
 * @version
 */

const SSO = require('../lib/sso');
module.exports = app => {

    class Customer extends app.BaseService {

        /**
         * 构造函数
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        constructor(ctx) {
            super(ctx);
            this.tableName = 'customer';
        }

        /**
         * 查询过虑
         *
         * @param {Object} data - 筛选表单中的get数据
         * @return {void}
         */
        searchFilter(data) {
            this.initSqlBuilder();

            // 企业类型筛选
            if(data.company_type !== undefined && data.company_type !== '') {
                let companyType = data.company_type;
                if (isNaN(companyType)) {
                    return;
                }

                this.sqlBuilder.setAndWhere('company_type', {
                    value: companyType,
                    operate: '=',
                });
            }

            // 省份筛选
            if(data.province !== undefined && data.province !== '') {
                let province = data.province;
                if (isNaN(province)) {
                    return;
                }

                this.sqlBuilder.setAndWhere('province', {
                    value: province,
                    operate: '=',
                });
            }

            // 最后登录时间筛选
            if (data.last_login !== undefined && data.last_login !== '') {
                let startTime = data.last_login + ' 00:00:00';
                let endTime = data.last_login + ' 23:59:59';

                startTime = Date.parse(new Date(startTime)) / 1000;
                endTime = Date.parse(new Date(endTime)) / 1000;

                if (isNaN(startTime) || isNaN(endTime) || startTime > endTime) {
                    return;
                }

                this.sqlBuilder.setAndWhere('last_login', {
                    value: startTime,
                    operate: '>=',
                });

                this.sqlBuilder.setAndWhere('last_login', {
                    value: endTime,
                    operate: '<=',
                });
            }

            const searchType = data.search_type === undefined ? 0 : parseInt(data.search_type);
            if (data.keyword !== undefined && data.keyword !== '') {
                switch (searchType) {
                    case 1:
                        // 通行账号手机
                        this.sqlBuilder.setAndWhere('mobile', {
                            value: this.db.escape(data.keyword + '%'),
                            operate: 'like',
                        });
                        break;
                    case 2:
                        break;
                    case 3:
                        break;
                    default:
                        break;
                }
            }

        }

        /**
         * 新增或修改SSO用户
         *
         * @param {Object} data - 接口返回的数据
         * @param {int} type - 接口调用类型
         * @return {Number} - 返回新增数据的id
         */
        async saveSSOUser(data) {
            let result = 0;
            const customerData = await this.db.get(this.tableName, { sso_id: data.sso_id });
            if (customerData !== null) {
                // 存在则更新表并返回id
                const updateData = {
                    id: customerData.id,
                    sso_id: data.sso_id,
                    email: data.email,
                    mobile: data.mobile,
                    name: data.name,
                };
                await this.db.update(this.tableName, updateData);
                return customerData.id;
            }
            const insertData = {
                sso_id: data.sso_id,
                email: data.email,
                mobile: data.mobile,
                name: data.name,
                create_time: new Date().getTime() / 1000,
            };
            result = await this.db.insert(this.tableName, insertData);

            return result.insertId;
        }

        /**
         * 查找多个表和SSO里是否有此用户
         *
         * @param {Object} account - 邮箱或手机号
         * @param {Object} pid - 项目id
         * @return {Number} - 返回值代表它们所属哪个表
         */
        async getDataInfo(account) {
            const result = null;

            // 先查询sso表
            const customerData = await this.getDataByAccount(account);

            if (customerData !== null) {
                return { type: 1, data: customerData };
            }

            // 没有再才查询sso用户接口
            const sso = new SSO(this.ctx);
            const SsoUserInfoData = await sso.getSSOUserData(account);

            if (SsoUserInfoData !== null) {
                const returnSSOData = {
                    sso_id: SsoUserInfoData.id,
                    name: SsoUserInfoData.username,
                    email: SsoUserInfoData.useremail,
                    mobile: SsoUserInfoData.mobile
                };
                return { type: 2, data: returnSSOData };
            }

            return result;
        }

        /**
         * 根据邮箱或手机号查找用户数据
         * @param account
         * @return {Promise.<*>}
         */
        async getDataByAccount(account) {
            this.initSqlBuilder();
            this.sqlBuilder.setOrWhere('email', {
                value: this.db.escape(account),
                operate: '=',
            });
            this.sqlBuilder.setOrWhere('mobile', {
                value: this.db.escape(account),
                operate: '=',
            });
            const [sql, sqlParam] = this.sqlBuilder.build(this.tableName, 'select');
            const info = await this.db.queryOne(sql, sqlParam);

            return info;
        }

    }

    return Customer;
};
