'use strict';

/**
 * 项目数据模型
 *
 * @author CaiAoLin
 * @date 2017/11/8
 * @version
 */

const cldOfficeConst = require('../const/cld_office');
module.exports = app => {

    class Enterprise extends app.BaseService {

        /**
         * 构造函数
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        constructor(ctx) {
            super(ctx);
            this.tableName = 'enterprise';
            // 状态相关
            this.status = {
                NORMAL: 1,
                DISABLE: 2,
            };
            this.displayStatus = [];
            this.displayStatus[this.status.NORMAL] = '正常';
            this.displayStatus[this.status.DISABLE] = '停用';

            this.statusClass = [];
            this.statusClass[this.status.NORMAL] = 'success';
            this.statusClass[this.status.DISABLE] = 'danger';
        }

        /**
         * 数据规则
         *
         * @param {String} name - 规则名称
         * @return {Object} - 返回数据规则
         */
        rule(name) {
            let rule = {};
            switch (name) {
                case 'enterprise':
                    rule = {
                        name: { type: 'string', required: true, min: 2 },
                        remark: { type: 'string', required: false, allowEmpty: true },
                        max_project: { type: 'string', required: true, min: 1 },
                        office: { type: 'string', required: true },
                    };
                    break;
                case 'enterpriseModify':
                    rule = {
                        name: { type: 'string', required: true, min: 2 },
                        // max_project: { type: 'string', required: true, min: 1 },
                        remark: { type: 'string', required: false, allowEmpty: true },
                        status: { type: 'string', required: true, max: 1 },
                        user_account: { type: 'string', required: false, allowEmpty: true },
                        office: { type: 'string', required: true },
                    };
                    break;
                case 'enterpriseSetting':
                    rule = {
                        max_project: { type: 'string', required: true, min: 1 },
                    };
                    break;
                default:
                    break;
            }
            return rule;
        }

        /**
         * 查询过滤
         *
         * @param {Object} data - 筛选表单中的get数据
         * @return {void}
         */
        searchFilter(data) {
            this.initSqlBuilder();

            if (data.office !== undefined && data.office !== '') {
                let office = data.office;
                if (isNaN(office)) {
                    return;
                }
                this.sqlBuilder.setAndWhere('office', {
                    value: office,
                    operate: '=',
                });
            }

            if (data.keyword === undefined || data.keyword === '') {
                return;
            }

            data.type = parseInt(data.type);
            // 过滤数据
            switch (data.type) {
                // 企业编号
                case 1:
                    this.sqlBuilder.setAndWhere('code', {
                        value: this.db.escape(data.keyword + '%'),
                        operate: 'like',
                    });
                    break;
                // 企业名称
                case 2:
                    this.sqlBuilder.setAndWhere('name', {
                        value: this.db.escape(data.keyword + '%'),
                        operate: 'like',
                    });
                    break;
                default:
                    break;
            }

        }

        /**
         * 保存设置数据
         *
         * @param {Object} data - 表单post数据
         * @param {Number} id - 修改id
         * @return {boolean} - 返回执行结果
         */
        async saveSetting(data, id = 0) {
            id = parseInt(id);
            delete data._csrf;

            data.max_project = parseInt(data.max_project);
            data.id = id;
            delete data.create_time;

            const operate = await this.db.update(this.tableName, data);

            return operate.affectedRows > 0;
        }

        /**
         * 保存数据
         *
         * @param {Object} data - 表单post数据
         * @param {Number} id - 用于判断修改还是新增的id
         * @return {boolean} - 返回执行结果
         */
        async save(data, id = 0) {
            let result = false;
            id = parseInt(id);
            delete data._csrf;

            // 初始化事务
            this.transaction = await this.db.beginTransaction();
            try {
                if (id > 0) {
                    data.id = id;
                    delete data.create_time;
                } else {
                    // 生成企业code 格式：C181101 取年月然后加counter
                    const current = new Date();
                    const year = current.getFullYear().toString().slice(2);
                    const month = current.getMonth() + 1 < 10 ? '0' + (current.getMonth() + 1) : current.getMonth() + 1;
                    const counterKey = year + month + 'counter';
                    let counter = await this.cache.incr(counterKey);
                    counter = counter < 10 ? '0' + counter : counter;
                    data.code = 'C' + year + month + counter;

                    // 获取当前创建者数据
                    const sessionUserData = this.ctx.session.managerSession;
                    const officeString = cldOfficeConst.list[sessionUserData.office] === undefined ?
                        '超级管理员' : cldOfficeConst.list[sessionUserData.office];
                    data.creator = officeString + '-' + sessionUserData.username;

                    // 默认状态为正常
                    data.status = this.status.NORMAL;

                }

                const operate = id === 0 ? await this.transaction.insert(this.tableName, data) :
                    await this.transaction.update(this.tableName, data);

                if (operate.affectedRows <= 0) {
                    throw '保存项目数据失败';
                }

                // 新增企业账号
                if (data.user_account && data.user_account !== '') {
                    const enterpriseId = id === 0 ? operate.insertId : id;
                    this.ctx.service.projectAccount.transaction = this.transaction;

                    // 先判断表中是否已存在该账号
                    const enterpriseAccountData = await this.ctx.service.projectAccount.isEnterpriseAccountExist(data.user_account, enterpriseId);

                    if (enterpriseAccountData !== null) {
                        await this.ctx.service.projectAccount.checkAdminEnterpriseAccount(enterpriseAccountData);
                    } else {
                        // 再判断账号属于customer还是sso
                        const result = await this.ctx.service.customer.getDataInfo(data.user_account);
                        if (result !== null && result.type === 2) {
                            // 添加到customer表
                            this.ctx.service.customer.transaction = this.transaction;
                            await this.ctx.service.customer.saveSSOUser(result.data);
                        }
                        this.ctx.service.projectAccount.transaction = this.transaction;
                        await this.ctx.service.projectAccount.saveAdminEnterpriseAccount(result.data, enterpriseId);
                    }
                }

                result = await this.transaction.commit();

            } catch (error) {
                console.log(error);
                // 回滚
                await this.transaction.rollback();
            }

            return result;
        }

        /**
         * 假删除
         *
         * @param {Number} id - 删除的id
         * @return {Boolean} - 删除结果
         */
        async deleteEnterpriseById(id) {
            const updateData = {
                status: this.status.DISABLE,
                id,
            };

            const result = this.db.update(this.tableName, updateData);

            return result.affectedRows > 0;
        }
    }

    return Enterprise;
};
