'use strict';

/**
 * 后台用户数据模型
 *
 * @author CaiAoLin
 * @date 2017/8/29
 * @version
 */

// 加密类
const crypto = require('crypto');
// CLD相关
const Cld = require('../lib/cld');
const cldOfficeConst = require('../const/cld_office');
module.exports = app => {

    class Manager extends app.BaseService {

        /**
         * 构造函数
         *
         * @param {Object} ctx - egg全局context
         * @return {void}
         */
        constructor(ctx) {
            super(ctx);
            this.tableName = 'manager';
        }

        /**
         * 校验规则
         *
         * @param {String} name - 规则名称
         * @return {Object} - 返回对应的规则
         */
        rule(name) {
            let rule = {};
            switch (name) {
                case 'login':
                    rule = {
                        username: { type: 'string', required: true, allowEmpty: false, min: 2 },
                        password: { type: 'string', required: true, allowEmpty: false, min: 4 },
                    };
                    break;
                case 'add':
                    rule = {
                        username: { type: 'string', required: true, allowEmpty: false, min: 2, message: '请填写登录用户名' },
                        group_id: { type: 'string', required: true, allowEmpty: false, message: '请选择用户组' },
                        real_name: { type: 'string', required: true, allowEmpty: false, min: 2, message: '请填写真实姓名' },
                        telephone: { type: 'mobile', message: '请填写联系电话' },
                    };
                    break;
                case 'modify':
                    rule = {
                        username: { type: 'string', required: true, allowEmpty: false, min: 2, message: '请填写登录用户名' },
                        group_id: { type: 'string', required: true, allowEmpty: false, message: '请选择用户组' },
                        real_name: { type: 'string', required: true, allowEmpty: false, min: 2, message: '请填写真实姓名' },
                        telephone: { type: 'mobile', message: '请填写联系电话' },
                    };
                    break;
                case 'profile':
                    rule = {
                        password: { type: 'password', allowEmpty: true, min: 4 },
                        new_password: { type: 'password', allowEmpty: true, min: 6 },
                        confirm_password: { type: 'password', allowEmpty: true, min: 6, compare: 'new_password' },
                        real_name: { type: 'string', required: true, allowEmpty: false, min: 2 },
                        telephone: { type: 'mobile', message: '请填写联系电话' },
                    };
                    break;
                default:
                    break;
            }

            return rule;
        }

        /**
         * 查找数据
         *
         * @param {Object} data - 筛选表单中的get数据
         * @return {void}
         */
        searchFilter(data, managerSession) {
            this.initSqlBuilder();
            this.sqlBuilder.columns = ['id', 'username', 'real_name', 'create_time', 'last_login', 'login_ip',
                'group_id', 'token', 'can_login'];

            data.type = parseInt(data.type);
            if (data.keyword !== undefined) {
                switch (data.type) {
                    // 用户名
                    case 1:
                        this.sqlBuilder.setAndWhere('username', {
                            value: this.db.escape(data.keyword + '%'),
                            operate: 'like',
                        });
                        break;
                    // 姓名
                    case 2:
                        this.sqlBuilder.setAndWhere('real_name', {
                            value: this.db.escape(data.keyword + '%'),
                            operate: 'like',
                        });
                        break;
                    // 联系电话
                    case 3:
                        this.sqlBuilder.setAndWhere('telephone', {
                            value: this.db.escape(data.keyword + '%'),
                            operate: 'like',
                        });
                        break;
                    default:
                        break;
                }
            }

            // 办事处筛选
            if (parseInt(managerSession.office) !== 12) {
                this.sqlBuilder.setAndWhere('office', {
                    value: this.db.escape(managerSession.office),
                    operate: '=',
                });
            } else if (data.office !== undefined && data.office !== '') {
                this.sqlBuilder.setAndWhere('office', {
                    value: this.db.escape(data.office),
                    operate: '=',
                });
            }
        }

        /**
         * 用户验证
         *
         * @param {String} username - 用户名
         * @param {String} password - 未加密的密码
         * @return {boolean} - 判断结果
         */
        async validManager(username, password) {
            let result = false;

            // 接口验证CLD用户
            if (username !== 'admin') {
                // 验证成功后会在本地库中插入用户数据
                const cld = new Cld(this.ctx);
                result = await cld.loginValid(username, password);
            }
            const sql = 'SELECT m.*,g.permission FROM ' + this.tableName + ' AS m ' +
                'LEFT JOIN ' + this.ctx.service.group.tableName + ' AS g ON m.group_id = g.id ' +
                'WHERE username=?';
            const manager = await this.db.queryOne(sql, [username]);
            if (manager === null) {
                return false;
            }
           
            // 判断是否被禁止登陆
            if (manager.can_login !== 1) {
                throw '用户被禁用';
            }
            // 验证本地用户
            if (username === 'admin') {
                const encryptPassword = crypto.createHmac('sha1', manager.token).update(password)
                    .digest().toString('base64');
                result = encryptPassword === manager.password;
            }

            if (result) {
                // 成功后写入session
                const currentTime = new Date().getTime();
                const sessionToken = crypto.createHmac('sha1', currentTime + '').update(username)
                    .digest().toString('base64');
                const managerSession = {
                    username: manager.username,
                    loginTime: currentTime,
                    sessionToken,
                    userID: manager.id,
                    permission: manager.permission,
                    office: manager.office,
                    avatar: manager.avatar !== null ? 'http://cld.smartcost.com.cn' + manager.avatar + '_1.jpg' : null,
                };
                this.ctx.session.managerSession = managerSession;

                // 更改登录数据
                const updateData = {
                    id: manager.id,
                    last_login: new Date().getTime() / 1000,
                    login_ip: this.ctx.request.ip,
                };
                await this.db.update(this.tableName, updateData);
            }

            return result;
        }

        /**
         * 保存用户数据
         *
         * @param {Object} data - post过来的数据
         * @param {Number} id - 保存的id
         * @return {boolean} - 保存结果
         */
        async save(data, id) {
            delete data._csrf;
            id = parseInt(id);
            id = isNaN(id) ? 0 : id;

            if (id > 0) {
                // 修改操作
                data.id = id;
                const managerData = await this.getDataById(id);
                if (managerData === null) {
                    throw '不存在用户';
                }
                data.token = managerData.token;
                delete data.create_time;

                if (data.password === '' || data.password === undefined) {
                    // 密码为空则删除
                    delete data.password;
                } else {
                    // 处理密码
                    data.password = crypto.createHmac('sha1', data.token).update(data.password)
                        .digest().toString('base64');
                }

                // 修改密码要通过原密码的验证
                if (data.new_password !== '' && data.new_password !== undefined) {
                    if (data.password !== managerData.password) {
                        throw '用户密码错误';
                    }
                    data.password = crypto.createHmac('sha1', data.token).update(data.new_password)
                        .digest().toString('base64');
                }
                delete data.new_password;
                delete data.confirm_password;
            } else {
                // 新增则生成token
                data.token = this.ctx.helper.generateRandomString(6);
            }

            const operate = id === 0 ? await this.db.insert(this.tableName, data) :
                await this.db.update(this.tableName, data);

            const result = operate.affectedRows > 0;
            return result;
        }

        /**
         * 新增一个CLD用户
         *
         * @param {Object} data - 用户数据
         * @return {boolean} - 新增结果
         */
        async addCldUser(data) {
            let result = false;
            if (data.username === undefined) {
                return result;
            }
            const condition = {
                username: data.username,
                token: 'CLD',
            };
            const office = parseInt(data.office) === 12 && parseInt(data.departmentID) === 1 ? cldOfficeConst.type.YJZX : data.office;
            const category = parseInt(office) === cldOfficeConst.type.YJZX ? cldOfficeConst.list[cldOfficeConst.type.YJZX] : data.category;

            // 先查找是否已经存在
            const managerData = await this.db.get(this.tableName, condition);
            if (managerData !== null) {
                result = true;
                // 更新已存在cld用户的数据
                const updateData = {
                    id: managerData.id,
                    office,
                    category,
                    email: data.email,
                    qq: data.qq,
                    fixedphone: data.fixedphone,
                    telephone: data.telephone,
                    position: data.position,
                    avatar: data.avatar,
                };
                await this.db.update(this.tableName, updateData);
                return result;
            }

            // 不存在则插入数据库
            const insertData = {
                username: data.username,
                password: 'CLD Password',
                create_time: new Date().getTime() / 1000,
                group_id: 0,
                token: 'CLD',
                office,
                category: data.category,
                email: data.email,
                qq: data.qq,
                fixedphone: data.fixedphone,
                telephone: data.telephone,
                position: data.position,
                avatar: data.avatar,
            };
            result = await this.db.insert(this.tableName, insertData);

            return result;
        }

        /**
         * 获取用户列表
         *
         * @param {String} indexBy - 按某个字段组合数据
         * @param {Boolean} isCache - 是否查找缓存
         * @return {Object} 返回用户数据
         */
        async getManagerList(indexBy, isCache = true) {
            const cacheKey = 'managerList:' + indexBy;
            let data = this.cache.get(cacheKey);
            if (data !== null && isCache) {
                data = JSON.parse(data);
                return data;
            }

            const managerData = await this.db.select(this.tableName);
            if (managerData.length === 0) {
                return {};
            }

            const result = {};
            for (const manager of managerData) {
                if (result[manager[indexBy]] === undefined) {
                    result[manager[indexBy]] = [];
                }
                const tmp = {
                    id: manager.id,
                    username: manager.username,
                };
                result[manager[indexBy]].push(tmp);
            }

            if (Object.keys(result).length > 0) {
                this.cache.set(cacheKey, JSON.stringify(result), 'EX', app.config.cacheTime);
            }

            return result;
        }

        /**
         * 获取用户权限
         *
         * @param {String} userID - 用户ID
         * @return {Object} 返回用户权限数据
         */
        async getManagerPermission(userID) {
            const sql = 'SELECT m.*,g.permission FROM ' + this.tableName + ' AS m ' +
                'LEFT JOIN ' + this.service.group.tableName + ' AS g ON m.group_id = g.id ' +
                'WHERE m.id=?';
            const manager = await this.db.queryOne(sql, [userID]);
            return manager.permission;
        }
    }


    return Manager;
};
