'use strict';

/**
 * 权限相关业务逻辑
 *
 * @author CaiAoLin
 * @date 2017/9/27
 * @version
 */

module.exports = app => {

    class Permission extends app.BaseService {
        /**
         * 构造函数
         *
         * @param {Object} ctx - egg全局context
         * @return {void}
         */
        constructor(ctx) {
            super(ctx);
            this.tableName = 'permission';
        }

        /**
         * 校验规则
         *
         * @return {Object} - 返回规则数据
         */
        rule() {
            const rule = {
                name: { type: 'string', required: true, allowEmpty: false },
                controller: { type: 'string', allowEmpty: true, required: false },
                action: { type: 'string', allowEmpty: true, required: false },
                icon_class: { type: 'string', allowEmpty: true, required: false },
                pid: { type: 'string', required: true },
            };
            return rule;
        }

        /**
         * 获取菜单数据
         *
         * @param {boolean} isCache - 是否使用缓存(默认不使用)
         * @return {Array} - 返回整理好的数据
         */
        async getMenu(isCache = false) {
            const cacheKey = 'allMenu';

            const cacheData = await this.cache.get(cacheKey);
            if (cacheData !== null && isCache) {
                return JSON.parse(cacheData);
            }
            let result = [];
            // 获取所有数据
            const dataList = await this.getAllData();
            
            if (dataList === null || dataList.length <= 0) {
                return result;
            }
            // 先整理出一级菜单
            const sourceData = {};
            for (const index in dataList) {
                // 以父id为index 分组对象
                if (sourceData[dataList[index].pid] === undefined) {
                    sourceData[dataList[index].pid] = [];
                }
                sourceData[dataList[index].pid].push(dataList[index]);
            }
            result = sourceData[0];
            delete sourceData[0];

            do {
                this._sort(result, sourceData);
            } while (Object.keys(sourceData).length > 0);
            
            // 加入缓存
            if (result.length > 0) {
                this.cache.set(cacheKey, JSON.stringify(result), 'EX', app.config.cacheTime);
            }
            return result;
        }

        /**
         * 获取所有数据
         *
         * @param {boolean} isCache - 是否使用缓存(默认不使用)
         * @param {boolean} indexById - 组合字段
         * @return {Array} - 返回的数据
         */
        async getAllData(isCache = false, indexById = false) {
            const cacheKey = indexById ? 'allPermissionByIndex' : 'allPermission';
            const cacheData = await this.cache.get(cacheKey);
            if (cacheData !== null && isCache) {
                return JSON.parse(cacheData);
            }

            let dataList = await this.db.select(this.tableName, {
                orders: [['create_time', 'desc']],
            });
            if (indexById && dataList.length > 0) {
                const tmpData = {};
                for (const data of dataList) {
                    tmpData[data.id] = data;
                }
                dataList = tmpData;
            }
            // 加入缓存
            if (dataList.length !== null) {
                this.cache.set(cacheKey, JSON.stringify(dataList), 'EX', app.config.cacheTime);
            }
            return dataList;
        }

        /**
         * 递归组成数据
         *
         * @param {Object} topLevel - 顶层数据
         * @param {Object} sourceData - 原数据
         * @return {void}
         */
        async _sort(topLevel, sourceData) {
            for (const index in topLevel) {
                if (topLevel[index].children !== undefined) {
                    this._sort(topLevel[index].children, sourceData);
                }
                if (!sourceData.hasOwnProperty(topLevel[index].id)) {
                    continue;
                }
                if (topLevel[index].children === undefined) {
                    topLevel[index].children = [];
                }
                topLevel[index].children = sourceData[topLevel[index].id];
                delete sourceData[topLevel[index].id];
            }
        }

        /**
         * 保存数据
         *
         * @param {Object} data - post过来的数据
         * @param {Number} id - 更新的id
         * @return {boolean} - 保存结果
         */
        async save(data, id = 0) {
            delete data._csrf;

            if (id > 0) {
                data.id = id;
            }
            data.icon_class = data.iconClass;
            delete data.iconClass;

            const operate = id === 0 ? await this.db.insert(this.tableName, data) :
                await this.db.update(this.tableName, data);

            const result = operate.affectedRows > 0;
            // 保存成功则更新缓存
            if (result) {
                this.flushCache();
            }
            return result;
        }

        /**
         * 获取当前id对应的顶级父类id
         *
         * @param {Number} id - 当前权限id
         * @param {Object} allPermission - 所有权限列表
         * @return {void}
         */
        getTopPid(id, allPermission) {
            if (allPermission[id] !== undefined && allPermission[id].pid !== 0) {
                this.getTopPid(allPermission[id].pid, allPermission);
            } else {
                this.ctx.permissionRootId = id;
            }
        }

        /**
         * 根据id获取对应名称
         *
         * @param {String|Number} id - 可获取单个或批量id
         * @return {String} - 返回名称
         */
        async getNameById(id) {
            const idList = typeof id === 'string' ? id.split(',') : [id];

            if (idList.length <= 0) {
                return '';
            }

            const permissionData = await this.getAllData(false, true);
            const tmpList = [];
            for (const id of idList) {
                if (permissionData[id] !== undefined) {
                    tmpList.push(permissionData[id].name);
                }
            }

            return tmpList.join(',');
        }

        async getCacheDataById(id) {
            const permissionData = await this.getAllData(false, true);
            return permissionData[id];
        }

        /**
         * 根据权限显示菜单
         *
         * @param {Array} menuData - 数据库中获取的菜单数据
         * @param {boolean} isCache - 是否缓存(默认关闭)
         * @return {Array} - 返回数据
         */
        async sortMenuByPermission(menuData, isCache = false) {
            const result = [];
            const managerSession = this.ctx.session.managerSession;
            if (managerSession.userID === undefined) {
                return result;
            }

            // 先取缓存
            const cacheKey = 'permissionMenu:' + managerSession.userID;
            let data = await this.cache.get(cacheKey);

            if (isCache && data !== null) {
                data = JSON.parse(data);
                return data;
            }
            
            // 获取权限数据
            // let permissionList = managerSession.permission;
            let permissionList = await this.ctx.service.manager.getManagerPermission(managerSession.userID);

            if (permissionList === '' || permissionList === null) {
                return result;
            }
            // 如果是超级管理员则直接返回
            if (permissionList === 'all') {
                return menuData;
            }

            permissionList = permissionList.split(',');
            // 递归过滤数据
            this._recursionFilter(menuData, permissionList);

            return menuData;
        }

        /**
         * 递归筛选
         *
         * @param {Array} menuData - 数据库中获取的菜单数据
         * @param {Array} permission - 权限列表
         * @param {Number} level - 层级(用于调试)
         * @return {Array} - 返回过滤好的数据
         */
        _recursionFilter(menuData, permission, level = 1) {
            for (const index in menuData) {
                if (permission.indexOf(menuData[index].id + '') < 0) {
                    delete menuData[index];
                    continue;
                }
                if (menuData[index].children === undefined || menuData[index].children.length <= 0) {
                    continue;
                }

                menuData[index].children = this._recursionFilter(menuData[index].children, permission, level + 1);
            }
            return menuData;
        }

        /**
         * 更新缓存
         *
         * @return {void}
         */
        flushCache() {
            const menuCacheKey = 'allMenu';
            const permissionCacheKey = 'allPermission';
            const permissionIndexCacheKey = 'allPermissionByIndex';
            this.cache.del(menuCacheKey);
            this.cache.del(permissionCacheKey);
            this.cache.del(permissionIndexCacheKey);
        }

    }

    return Permission;
};
