'use strict';

/**
 * 项目账号数据模型
 *
 * @author CaiAoLin
 * @date 2017/11/13
 * @version
 */

// 加密类
const crypto = require('crypto');
const SMS = require('../lib/sms');
const SmsAliConst = require('../const/sms_alitemplate');
module.exports = app => {

    class ProjectAccount extends app.BaseService {

        /**
         * 构造函数
         *
         * @param {Object} ctx -egg全局变量
         * @return {void}
         */
        constructor(ctx) {
            super(ctx);
            this.tableName = 'project_account';
        }

        /**
         * 数据规则
         *
         * @param {String} name - 场景名称
         * @return {Object} -返回数据规则
         */
        rule(name) {
            let rule = {};
            switch (name) {
                case 'add':
                    rule = {
                        account_group: { type: 'string', required: true },
                        account: { type: 'string', required: true, min: 2 },
                        password: { type: 'string', required: true, min: 6 },
                        name: { type: 'string', required: true, min: 2 },
                        company: { type: 'string', required: true, min: 2 },
                        role: { type: 'string', allowEmpty: true, min: 2 },
                        mobile: { type: 'mobile', allowEmpty: true },
                    };
                    break;
                case 'modify':
                    rule = {
                        account_group: { type: 'string', required: true },
                        // account: { type: 'string', required: true, min: 2 },
                        name: { type: 'string', required: true, min: 2 },
                        company: { type: 'string', required: true, min: 2 },
                        role: { type: 'string', allowEmpty: true, min: 2 },
                        mobile: { type: 'mobile', allowEmpty: true },
                    };
                    break;
                default:
                    break;
            }
            return rule;
        }

        /**
         * 查询过虑
         *
         * @param {Object} data - 筛选表单中的get数据
         * @return {void}
         */
        searchFilter(data, projectId) {
            this.initSqlBuilder();

            this.sqlBuilder.columns = ['id', 'project_id', 'account', 'account_group', 'name', 'company', 'last_login', 'role', 'mobile', 'telephone', 'is_admin', 'enable', 'auth_mobile', 'bind'];
            // 名字筛选
            if (data.account_group !== undefined && data.account_group !== '' && parseInt(data.account_group) !== 0) {
                this.sqlBuilder.setAndWhere('account_group', {
                    value: parseInt(data.account_group),
                    operate: '=',
                });
            }

            // 名字筛选
            if (data.keyword !== undefined && data.keyword !== '') {
                this.sqlBuilder.setAndWhere('name', {
                    value: this.db.escape(data.keyword + '%'),
                    operate: 'like',
                });
            }

            this.sqlBuilder.setAndWhere('project_id', {
                value: projectId,
                operate: '=',
            });
        }

        /**
         * 根据项目ID获取用户列表
         *
         * @param {Number} projectId -项目id
         * @return {Array} - 返回用户列表
         */
        async getAccountByProjectId(projectId) {
            if (this.sqlBuilder === null) {
                return null;
            }

            this.sqlBuilder.setAndWhere('project_id', {
                value: projectId,
                operate: '=',
            });

            const [sql, sqlParam] = this.sqlBuilder.build(this.tableName);
            const result = this.db.query(sql, sqlParam);

            return result;
        }

        /**
         * 保存数据
         *
         * @param {Object} data - 表单post数据
         * @param {Number} id - 用于判断修改还是新增的id
         * @return {boolean} - 返回执行结果
         */
        async save(data, id = 0) {
            id = parseInt(id);
            delete data._csrf;
            delete data.auth_mobile;

            data.project_id = this.ctx.session.sessionProjectData.id;

            // 初始化事务
            this.transaction = await this.db.beginTransaction();

            let result = false;
            try {
                if (id > 0) {
                    // 修改操作时
                    delete data.create_time;
                    data.id = id;
                } else {
                    // 重名检测
                    const accountData = await this.db.select(this.tableName, {
                        where: {
                            account: data.account,
                            project_id: data.project_id,
                        },
                    });
                    if (accountData.length > 0) {
                        throw '已存在对应的帐户名';
                    }

                    // 加密密码
                    data.password = crypto.createHmac('sha1', data.account).update(data.password)
                        .digest().toString('base64');
                }

                const adminData = await this.getDataByCondition({
                    is_admin: 1,
                    project_id: data.project_id,
                });
                // 令其它用户设为非管理员
                if (data.is_admin !== undefined) {
                    if (adminData !== null) {
                        const updateAccountData = {
                            is_admin: 0,
                            id: adminData.id,
                        };
                        await this.transaction.update(this.tableName, updateAccountData);
                    }

                    const updateProjectData = {
                        id: data.project_id,
                        user_account: data.account,
                    };
                    await this.transaction.update(this.ctx.service.project.tableName, updateProjectData);
                    // 后台设置为管理员时把账号启用
                    data.enable = 1;
                } else if (data.is_admin === undefined && adminData !== null && adminData.account === data.account) {
                    const updateProjectData = {
                        id: data.project_id,
                        user_account: '',
                    };
                    await this.transaction.update(this.ctx.service.project.tableName, updateProjectData);
                    data.is_admin = 0;
                }

                const operate = id === 0 ? await this.transaction.insert(this.tableName, data) :
                    await this.transaction.update(this.tableName, data);

                result = operate.affectedRows > 0;
                this.transaction.commit();
            } catch (error) {
                this.transaction.rollback();
            }
            return result;
        }

        /**
         * 判断是否存在对应的账号
         *
         * @param {String} account - 账号名称
         * @param {Number} projectId - 项目id
         * @return {Boolean} - 返回是否存在
         */
        async isAccountExist(account, projectId, extraId = '') {
            let result = true;
            const accountData = await this.db.get(this.tableName, { account, project_id: projectId });
            if (accountData) {
                if (accountData.id === parseInt(extraId)) {
                    result = false;
                }
            } else {
                result = false;
            }
            return result;
        }

        /**
         * 查找管理员并对比，不同则更新账号
         *
         * @param {String} account - 账号名称
         * @param {Number} projectId - 项目id
         * @return {Boolean} - 返回是否存在
         */
        async checkAdminProjectAccount(projectAccountData) {
            const accountData = await this.db.get(this.tableName, { project_id: projectAccountData.project_id, is_admin: 1 });
            if (accountData !== null && accountData.account === projectAccountData.account) {
                return true;
            } else if (accountData !== null && accountData.account !== projectAccountData.account) {
                // 撤销原有管理员身份
                const updateData = {
                    id: accountData.id,
                    is_admin: 0,
                };
                await this.transaction.update(this.tableName, updateData);
            }
            // 更新管理员身份
            const updateData2 = {
                id: projectAccountData.id,
                is_admin: 1,
            };
            const operate = await this.transaction.update(this.tableName, updateData2);

            const result = operate.affectedRows > 0;
            return result;
        }

        /**
         * 新增项目管理员账号
         *
         * @param {String} account - 新增账号
         * @param {Number} projectId - 项目id
         * @return {Boolean} - 新增结果
         */
        async saveAdminProjectAccount(data, projectId) {
            // 查找对应账号数据
            // console.log(data)
            // const accountData = await this.db.get(this.tableName, { project_id: projectId, is_admin: 1 });
            // let type = null;
            // if (accountData !== null && (accountData.account === data.email || accountData.account === data.mobile)) {
            //     return true;
            // } else if (accountData !== null && accountData.account !== data.email && accountData.account !== data.mobile) {
            //     // 撤销原有管理员身份
            //     const updateData = {
            //         id: accountData.id,
            //         is_admin: 0,
            //     };
            //     await this.transaction.update(this.tableName, updateData);
            // }
            // const account = data.email === null || data.email === '' ? data.mobile : data.email;

            // 新增数据
            const insertData = {
                project_id: projectId,
                account: data.account,
                password: 'JL password',
                account_group: data.account_group,
                name: data.name,
                mobile: data.mobile,
                company: data.company !== undefined ? data.company : '',
                is_admin: 1,
                role: '管理员',
                create_time: new Date().getTime() / 1000,
                // permission: 'all',
            };
            const result = await this.transaction.insert(this.tableName, insertData);
            return result;
            //return result.affectedRows > 0;
        }

        /**
         * 重置密码
         *
         * @param {Number} accountId - 账号id
         * @param {String} password - 重置的密码
         * @return {Boolean} - 重置结果
         */
        async resetPassword(accountId, password, account = '') {
            // 初始化事务
            this.transaction = await this.db.beginTransaction();
            let result = false;
            try {
                // 查找对应账号数据
                const accountData = await this.getDataByCondition({ id: accountId });
                if (accountData.account === undefined) {
                    throw '不存在对应账号';
                }
                // 加密密码
                const encryptPassword = account ? crypto.createHmac('sha1', account).update(password)
                    .digest().toString('base64') : crypto.createHmac('sha1', accountData.account).update(password)
                    .digest().toString('base64');
                // 更新账号密码
                if (account) {
                    const sql = 'UPDATE ?? SET account=?,password=? WHERE id=? AND password != ?;';
                    const sqlParam = [this.tableName, account, encryptPassword, accountId, 'SSO password'];
                    const operate = await this.transaction.query(sql, sqlParam);
                    result = operate.affectedRows > 0;
                } else {
                    const sql = 'UPDATE ?? SET password=? WHERE id=? AND password != ?;';
                    const sqlParam = [this.tableName, encryptPassword, accountId, 'SSO password'];
                    const operate = await this.transaction.query(sql, sqlParam);
                    result = operate.affectedRows > 0;
                }

                if (!result) {
                    throw '更新密码失败';
                }

                // 发送短信
                if (accountData.auth_mobile !== '') {
                    const sms = new SMS(this.ctx);
                    // const content = '【纵横计量支付】账号：' + (account ? account : accountData.account) + '，密码重置为：' + password;
                    // sms.send(accountData.auth_mobile, content);
                    sms.aliSend(accountData.auth_mobile, {
                        account: account ? account : accountData.account,
                        password,
                    }, SmsAliConst.template.mmcz);
                }

                this.transaction.commit();
            } catch (error) {
                this.transaction.rollback();
            }

            return result;
        }

        /**
         * 重置副密码
         *
         * @param {Number} accountId - 账号id
         * @param {String} password - 重置的副密码
         * @return {Boolean} - 重置结果
         */
        async resetBackdoorPassword(accountId, password) {
            // 初始化事务
            this.transaction = await this.db.beginTransaction();
            let result = false;
            try {
                // 查找对应账号数据
                const accountData = await this.getDataByCondition({ id: accountId });
                if (accountData.account === undefined) {
                    throw '不存在对应账号';
                }
                const sql = 'UPDATE ?? SET backdoor_password=? WHERE id=?';
                const sqlParam = [this.tableName, password, accountId];
                const operate = await this.transaction.query(sql, sqlParam);
                result = operate.affectedRows > 0;

                if (!result) {
                    throw '更新密码失败';
                }

                this.transaction.commit();
            } catch (error) {
                this.transaction.rollback();
            }

            return result;
        }



        /**
         * 判断企业是否存在对应的账号
         *
         * @param {String} account - 账号名称
         * @param {Number} projectId - 项目id
         * @return {Boolean} - 返回是否存在
         */
        async isEnterpriseAccountExist(account, enterpriseId) {
            const accountData = await this.db.get(this.tableName, { account: account, enterprise_id: enterpriseId });
            return accountData;
        }

        /**
         * 查找企业管理员并对比，不同则更新账号
         *
         * @param {String} account - 账号名称
         * @param {Number} projectId - 项目id
         * @return {Boolean} - 返回是否存在
         */
        async checkAdminEnterpriseAccount(enterpriseAccountData) {
            const accountData = await this.db.get(this.tableName, { enterprise_id: enterpriseAccountData.enterprise_id, is_admin: 1 });
            if (accountData !== null && accountData.account === enterpriseAccountData.account) {
                return true;
            } else if (accountData !== null && accountData.account !== enterpriseAccountData.account) {
                // 撤销原有管理员身份
                const updateData = {
                    id: accountData.id,
                    is_admin: 0,
                };
                await this.transaction.update(this.tableName, updateData);
            }
            // 更新管理员身份
            const updateData2 = {
                id: enterpriseAccountData.id,
                is_admin: 1,
            };
            const operate = await this.transaction.update(this.tableName, updateData2);

            const result = operate.affectedRows > 0;
            return result;
        }

        /**
         * 新增项目管理员账号
         *
         * @param {String} account - 新增账号
         * @param {Number} projectId - 项目id
         * @return {Boolean} - 新增结果
         */
        async saveAdminEnterpriseAccount(data, enterpriseId) {
            // 查找对应账号数据
            const accountData = await this.db.get(this.tableName, { enterprise_id: enterpriseId, is_admin: 1 });
            let type = null;
            if (accountData !== null && (accountData.account === data.email || accountData.account === data.mobile)) {
                return true;
            } else if (accountData !== null && accountData.account !== data.email && accountData.account !== data.mobile) {
                // 撤销原有管理员身份
                const updateData = {
                    id: accountData.id,
                    is_admin: 0,
                };
                await this.transaction.update(this.tableName, updateData);
            }

            const account = data.email === null || data.email === '' ? data.mobile : data.email;

            // 新增数据
            const insertData = {
                enterprise_id: enterpriseId,
                account: account,
                password: 'SSO password',
                account_group: data.account_group,
                name: data.name,
                mobile: data.mobile,
                company: data.company !== undefined ? data.company : '',
                is_admin: 1,
                role: '管理员',
                create_time: new Date().getTime() / 1000,
                // permission: 'all',
            };
            const result = await this.transaction.insert(this.tableName, insertData);
            return result.affectedRows > 0;
        }

        /**
         * 根据企业ID获取用户列表
         *
         * @param {Number} projectId -项目id
         * @return {Array} - 返回用户列表
         */
        async getAccountByEnterpriseId(enterpriseId) {
            if (this.sqlBuilder === null) {
                return null;
            }

            this.sqlBuilder.setAndWhere('enterprise_id', {
                value: enterpriseId,
                operate: '=',
            });

            const [sql, sqlParam] = this.sqlBuilder.build(this.tableName);
            const result = this.db.query(sql, sqlParam);

            return result;
        }

        /**
         * 保存数据
         *
         * @param {Object} data - 表单post数据
         * @param {Number} id - 用于判断修改还是新增的id
         * @return {boolean} - 返回执行结果
         */
        async enterpriseSave(data, id = 0) {
            id = parseInt(id);
            delete data._csrf;
            delete data.auth_mobile;

            data.enterprise_id = this.ctx.session.sessionEnterpriseData.id;
            if (id > 0) {
                // 修改操作时
                delete data.create_time;
                data.id = id;
            } else {
                // 重名检测
                const accountData = await this.db.select(this.tableName, {
                    where: {
                        account: data.account,
                        enterprise_id: data.enterprise_id,
                    },
                });
                if (accountData.length > 0) {
                    throw '已存在对应的帐户名';
                }

                // 加密密码
                data.password = crypto.createHmac('sha1', data.account).update(data.password)
                    .digest().toString('base64');

            }
            const operate = id === 0 ? await this.db.insert(this.tableName, data) :
                await this.db.update(this.tableName, data);

            const result = operate.affectedRows > 0;

            return result;
        }

    }

    return ProjectAccount;
};
