'use strict';

/**
 * 标准库基类
 *
 * @author CaiAoLin
 * @date 2018/1/10
 * @version
 */

const BaseTreeService = require('../base/base_tree_service');

class StandardLib extends BaseTreeService {

    /**
     * 构造函数
     *
     * @param {Object} ctx - egg全局变量
     * @param {String} tableName - 表名
     * @return {void}
     */
    constructor(ctx, setting, tableName) {
        super(ctx, setting);
        this.tableName = tableName;
        // 数据id（实际关联id，于默认自增id不同）
        this.dataId = '';
    }

    /**
     * 获取数据
     *
     * @param {Number} listId - 项目节列表id
     * @param {Number} level - 小于此层级的全部不显示
     * @return {Array} - 返回对应数据
     */
    // async getData(listId, level = 2) {
    //     this.initSqlBuilder();
    //     this.sqlBuilder.setAndWhere('list_id', {
    //         operate: '=',
    //         value: listId,
    //     });
    //     if (level !== -1) {
    //         this.sqlBuilder.setAndWhere('level', {
    //             operate: '<=',
    //             value: level,
    //         });
    //     }
    //     const [sql, sqlParam] = this.sqlBuilder.build(this.tableName);
    //     const list = await this.db.query(sql, sqlParam);
    //
    //     return list;
    // }

    /**
     * 转换数据
     *
     * @param {Array} list - 数据库查找的数据
     * @param {Boolean} tree - 是否展示为树形结构
     * @return {Object} - 返回转换后的数据
     */
    convertData(list, tree = true) {
        const rootData = [];
        const childData = {};
        let result = [];
        for (const tmp of list) {
            if (tmp.pid === 0 || tmp.pid === -1) {
                rootData.push(tmp);
                continue;
            }
            if (childData[tmp.pid] === undefined) {
                childData[tmp.pid] = [];
            }
            childData[tmp.pid].push(tmp);
        }
        // 递归组织数据
        if (tree) {
            this._recursionTreeData(rootData, childData);
        } else {
            this._recursionData(rootData, childData, result);
        }

        result = tree ? rootData : result;
        return result;
    }

    /**
     * 递归组织数组结构数据
     *
     * @param {Array} parent - 父节点
     * @param {Object} childData - 子元素(以pid为key的对象)
     * @param {Array} outputData - 处理后的数据
     * @return {void}
     */
    _recursionData(parent, childData, outputData = []) {
        for (const index in parent) {
            // 父节点先进数组
            outputData.push(parent[index]);
            // 判断是否存在子项
            if (childData[parent[index][this.dataId]] !== undefined) {
                // 子项作为父节点递归查找子项中是否还有子项
                const tmpParent = childData[parent[index][this.dataId]];
                // 已用的子项删除
                delete childData[parent[index][this.dataId]];
                // 递归元素
                this._recursionData(tmpParent, childData, outputData);
            }
        }
    }

    /**
     * 递归组织树结构数据
     *
     * @param {Array} parent - 父节点
     * @param {Object} childData - 子元素(以pid为key的对象)
     * @return {void}
     */
    _recursionTreeData(parent, childData = {}) {
        if (Object.keys(childData).length <= 0) {
            return;
        }
        for (const index in parent) {
            // 判断是否存在子项
            if (childData[parent[index][this.dataId]] !== undefined) {
                // 子项作为父节点递归查找子项中是否还有子项
                const tmpParent = childData[parent[index][this.dataId]];
                // 已用的子项删除
                delete childData[parent[index][this.dataId]];

                this._recursionTreeData(tmpParent, childData);
                // 递归完赋值回原数据
                parent[index].children = tmpParent;
            }
        }
    }

    /**
     * 导入数据
     *
     * @param {Object} excelData - excel导入的数据
     * @param {Number} listId -
     * @return {boolean} - 导入结果
     */
    async importData(excelData, listId) {
        let result = false;
        const limit = 30000;
        if (excelData.length <= 0) {
            return result;
        }
        if (excelData.length + 1 > limit) {
            throw '超出导入的数量，限制导入' + limit + '条数据!';
        }
        // 初始化事务
        this.transaction = await this.db.beginTransaction();
        try {
            const insertData = [];
            const dataList = {};
            for (const tmp of excelData) {
                // 判断是否为数据
                const id = parseInt(tmp[0]);
                if (isNaN(id) || id <= 0) {
                    continue;
                }
                // 处理数据
                const pid = parseInt(tmp[1]);
                const code = tmp[2] === null || tmp[2] === undefined ? '' : tmp[2].toString().trim();
                const name = tmp[3].toString().trim();
                const unit = tmp[4] === null || tmp[4] === undefined ? '' : tmp[4].toString().trim();
                const level = parseInt(tmp[5]);

                const tmpData = {
                    code, name, pid, unit, level,
                    list_id: listId,
                    has_child: 0,
                };
                tmpData[this.dataId] = id;

                dataList[id] = tmpData;
                // 判断是否有子项
                if (dataList[tmpData.pid] !== undefined) {
                    dataList[tmpData.pid].has_child = 1;
                }
            }

            // 整理成可插入的形式
            for (const index in dataList) {
                insertData.push(dataList[index]);
            }

            // 没有数据
            if (insertData.length <= 0) {
                return result;
            }
            // 先删除全部数据再插入
            await this.transaction.delete(this.tableName, { list_id: listId });
            await this.transaction.insert(this.tableName, insertData);

            result = await this.transaction.commit();
        } catch (error) {
            // 回滚
            await this.transaction.rollback();
            result = false;
        }

        return result;
    }

    /**
     * 解析json数据
     * @param jsonData
     * @param listId
     * @returns {*}
     */
    parseJsonData(jsonData, listId) {
        try {
            const insertData = [], limit = 30000;
            const parseData = JSON.parse(jsonData.replace(/[\n\r\t]/g, ''));
            if (!parseData || parseData.length < 0) return false;
            if (parseData.length > limit) throw '超出导入的数量，限制导入' + limit + '条数据!';

            for (const pd of parseData) {
                const data = { list_id: listId };
                data[this.setting.kid] = pd.id;
                data[this.setting.pid] = pd.pid;
                data[this.setting.order] = pd.order;
                data[this.setting.fullPath] = pd.full_path;
                data[this.setting.level] = pd.level;
                data[this.setting.isLeaf] = pd.is_leaf;
                data.code = pd.code;
                data.b_code = pd.b_code;
                data.name = pd.name;
                data.unit = pd.unit;
                insertData.push(data);
            }
            return insertData;
        } catch (err) {
            throw err;
        }
    }

    /**
     * 导入JSON数据
     * @param jsonData
     * @param listId
     * @returns {Promise<*>}
     */
    async importJsonData(jsonData, listId) {
        const insertData = this.parseJsonData(jsonData, listId);

        // 初始化事务
        this.transaction = await this.db.beginTransaction();
        try {
            await this.transaction.delete(this.tableName, {list_id: listId});
            await this.transaction.insert(this.tableName, insertData);
            return await this.transaction.commit();
        } catch (err) {
            await this.transaction.rollback();
            return false;
            throw err;
        }
    }

    async exportJsonData(listId) {
        const where = {};
        where[this.setting.mid] = listId;
        const result = await this.getAllDataByCondition({
            where: where,
            columns: [
                this.setting.kid, this.setting.pid,
                this.setting.order, this.setting.fullPath, this.setting.level, this.setting.isLeaf,
                'code', 'b_code', 'name', 'unit'
            ],
        });
        for (const r of result) {
            r.id = r[this.setting.kid];
            delete r[this.setting.kid];
        }
        return result;
    }

    /**
     * 获取导出的数据
     *
     * @param {Number} listId
     * @return {Array} - 返回数据
     */
    async getExportData(listId) {
        const commonColumn = ['id（从1开始）', '父id(没有则为0)', '项目节编号', '名称', '单位', '层级(从1开始)'];
        commonColumn[2] = this.tableName === 'zh_project_chapter' ? commonColumn[2] : '清单编号';

        const result = [commonColumn];
        const libData = await this.db.select(this.tableName, {
            where: { list_id: listId },
            orders: [[this.dataId, 'asc']],
        });

        if (libData.length <= 0) {
            return result;
        }

        // 整理数据
        for (const tmp of libData) {
            const lib = [tmp[this.dataId], tmp.pid, tmp.code, tmp.name, tmp.unit, tmp.level];
            result.push(lib);
        }

        return result;
    }
}

module.exports = StandardLib;
