'use strict';

/**
 * 白名单数据模型
 *
 * @author CaiAoLin
 * @date 2017/10/16
 * @version
 */

// 加密类
const crypto = require('crypto');
const version = '1.0.0.0';

module.exports = app => {

    class WhiteList extends app.BaseService {

        /**
         * 构造函数
         *
         * @param {Object} ctx ctx - egg全局context
         * @return {void}
         */
        constructor(ctx) {
            super(ctx);
            this.tableName = 'white_list';
            this.type = {
                BACKEND: 1,
                API: 2,
            };
            this.typeDisplay = [];
            this.typeDisplay[this.type.BACKEND] = '后台访问';
            this.typeDisplay[this.type.API] = '接口访问';
        }

        /**
         * 校验规则
         *
         * @return {Object} - 返回校验规则
         */
        rule() {
            const rule = {
                ip: { type: 'ip', allowEmpty: false },
                enable: { type: 'string', allowEmpty: false, required: true },
                type: { type: 'string', allowEmpty: false, required: true },
                remark: { type: 'string', allowEmpty: true },
            };
            return rule;
        }

        /**
         * 查找数据
         *
         * @param {Object} data - 搜索表单中的数据
         * @return {void}
         */
        searchFilter(data) {
            this.initSqlBuilder();
            this.sqlBuilder.columns = ['id', 'ip', 'type', 'enable', 'create_time', 'remark', 'token'];
            this.sqlBuilder.limit = app.config.pageSize;
            this.sqlBuilder.offset = app.config.pageSize * this.ctx.page;

            data.type = parseInt(data.type);
            if (!isNaN(data.type) && data.type > 0) {
                this.sqlBuilder.setAndWhere('type', {
                    value: data.type,
                    operate: '=',
                });
            }

        }

        /**
         * 保存白名单数据
         *
         * @param {Object} data - 表单post过来的数据
         * @param {Number} id - 区分新增还是修改的数据id
         * @return {boolean} 保存结果
         */
        async save(data, id) {
            delete data._csrf;
            id = parseInt(id);
            id = isNaN(id) ? 0 : id;
            // 接口访问才需要token
            switch (data.type) {
                // 后台访问
                case '1':
                    data.token = '';
                    break;
                case '2':
                    if (data.token !== '') {
                        // 如果修改其他数据则不用修改token字段
                        delete data.token;
                    } else {
                        data.token = crypto.createHmac('sha1', data.ip).update(data.create_time + '')
                            .digest().toString('base64');
                    }
                    break;
                default:
                    break;
            }
            if (id !== 0) {
                data.id = id;
                delete data.create_time;
            }

            const operate = id === 0 ? await this.db.insert(this.tableName, data) :
                await this.db.update(this.tableName, data);

            const result = operate.affectedRows > 0;
            if (result) {
                const cacheKey = 'allWhiteList';
                this.cache.del(cacheKey);
            }
            return result;
        }

        /**
         * 获取白名单数据
         *
         * @param {boolean} isCache - 是否缓存(默认缓存)
         * @return {Array} - 返回数据
         */
        async getAllData(isCache = true) {
            const cacheKey = 'allWhiteList' + version;
            let data = await this.cache.get(cacheKey);
            if (data !== null && isCache) {
                return JSON.parse(data);
            }
            const condition = {
                where: { enable: 1 },
            };
            data = await this.db.select(this.tableName, condition);
            if (data.length === 0) {
                return data;
            }

            // 整理数据
            const whiteListData = {};
            for (const whiteList of data) {
                whiteListData[whiteList.ip + ':' + whiteList.type] = whiteList.token;
            }
            this.cache.set(cacheKey, JSON.stringify(whiteListData), 'EX', app.config.cacheTime);

            return whiteListData;
        }

        /**
         * 根据id删除记录
         * @param {Int} id - id
         */
        async delById(id) {
            await this.db.delete(this.tableName, { id });
        }

    }

    return WhiteList;
};
